/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;

import de.frankmuenster.jameica.orverw.gui.input.TextStyle;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.system.Application;

/**
 * Settings Klasse fr den jameica.OrdnerVerwaltung Plugin
 *
 * @author Frank Mnstervb
 * @version 1.0
 */
public final class Settings {

    /** System settings von Jameica. */
    private static de.willuhn.jameica.system.Settings jameicaSettings = Application.getPluginLoader()
            .getPlugin(OrdnerVerwaltung.class).getResources().getSettings();

    /** Der DB Service. */
    private static DBService dbService;

    /**
     * Small helper function to get the database service.
     *
     * @return db service.
     * @throws RemoteException
     *             the remote exception
     */
    public static DBService getDBService() throws RemoteException {
        synchronized (DBService.class) {
            if (Settings.dbService == null) {
                try {
                    // We have to ask Jameica's ServiceFactory.
                    // If we are running in Client/Server mode and we are the
                    // client, the factory returns the remote dbService from the
                    // Jameica server.
                    // The name and class of the service is defined in
                    // plugin.xml
                    Settings.dbService = (DBService) Application.getServiceFactory().lookup(OrdnerVerwaltung.class,
                            "orVerwDB"); //$NON-NLS-1$
                } catch (final Exception e) { // NOPMD
                    throw new RemoteException(I18N.tr("Settings.dbServiceDown"), e); //$NON-NLS-1$
                }
            }
        }
        return Settings.dbService;
    }

    /**
     * Liefert die default Linienfarbe.
     *
     * @return die default Linienfarbe
     */
    public static Color getDefaultBarColor() {
        return new Color(GUI.getDisplay(), Settings.jameicaSettings.getRGB("orverw.barColor", new RGB(0, 0, 240))); //$NON-NLS-1$
    }

    /**
     * Liefert den default Texttype f&uuml;r den Inhalt.
     *
     * @return default Texttype f&uuml;r den Inhalt
     */
    public static TextStyle getDefaultContentStyle() {
        final TextStyle style = new TextStyle();
        style.setAlignment(Settings.jameicaSettings.getInt("orverw.contentAlignment", //$NON-NLS-1$
                SWT.LEFT));
        final Color color = new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.contentColor", new RGB(0, 0, 0))); //$NON-NLS-1$
        style.setTextColor(color);
        final Font font = new Font(GUI.getDisplay(), Settings.jameicaSettings.getFontData("orverw.contentFont", //$NON-NLS-1$
                new FontData()));
        style.setFont(font);
        return style;
    }

    /**
     * Liefert den default Texttype f&uuml;r den Fubereich.
     *
     * @return default Texttype f&uuml;r den Fubereich
     */
    public static TextStyle getDefaultFooterStyle() {
        final TextStyle style = new TextStyle();
        style.setAlignment(Settings.jameicaSettings.getInt("orverw.footerAlignment", //$NON-NLS-1$
                SWT.LEFT));
        final Color color = new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.footerColor", new RGB(0, 0, 0))); //$NON-NLS-1$
        style.setTextColor(color);
        final Font font = new Font(GUI.getDisplay(), Settings.jameicaSettings.getFontData("orverw.footertFont", //$NON-NLS-1$
                new FontData()));
        style.setFont(font);
        return style;
    }

    /**
     * Liefert die default Label Hintergrundfarbe.
     *
     * @return default Label Hintergrundfarbe
     */
    public static Color getDefaultLabelBackground() {
        return new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.labelBackground", new RGB(250, 250, 190))); //$NON-NLS-1$
    }

    /**
     * Liefert den default Texttype f&uuml;r die Register Beschreibung.
     *
     * @return default Texttype f&uuml;r die Register Beschreibung
     */
    public static TextStyle getDefaultRegisterContentStyle() {
        final TextStyle style = new TextStyle();
        Color color = new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.registerContentBackground", new RGB(230, 230, 230))); //$NON-NLS-1$
        style.setBackgroundColor(color);
        color = new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.registerContentColor", new RGB(0, 0, 0))); //$NON-NLS-1$
        style.setTextColor(color);
        final Font font = new Font(GUI.getDisplay(), Settings.jameicaSettings.getFontData("orverw.registerContentFont", //$NON-NLS-1$
                new FontData()));
        style.setFont(font);
        return style;
    }

    /**
     * Liefert den default Texttype f&uuml;r den Register Index.
     *
     * @return default Texttype f&uuml;r den Register Index
     */
    public static TextStyle getDefaultRegisterIndexStyle() {
        final TextStyle style = new TextStyle();
        Color color = new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.registerIndexBackground", new RGB(230, 230, 230)));//$NON-NLS-1$
        style.setBackgroundColor(color);
        color = new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.registerIndexColor", new RGB(0, 0, 0)));//$NON-NLS-1$
        style.setTextColor(color);
        final Font font = new Font(GUI.getDisplay(), Settings.jameicaSettings.getFontData("orverw.registerIndexFont", //$NON-NLS-1$
                new FontData()));
        style.setFont(font);
        return style;
    }

    /**
     * Liefert den default Texttype f&uuml;r den Label Titelbereich.
     *
     * @return default Texttype f&uuml;r den Label Titelbereich
     */
    public static TextStyle getDefaultTitleStyle() {
        final TextStyle style = new TextStyle();
        style.setAlignment(Settings.jameicaSettings.getInt("orverw.titleAlignment", //$NON-NLS-1$
                SWT.LEFT));
        final Color color = new Color(GUI.getDisplay(),
                Settings.jameicaSettings.getRGB("orverw.titleColor", new RGB(0, 0, 0)));//$NON-NLS-1$
        style.setTextColor(color);
        final Font font = new Font(GUI.getDisplay(),
                Settings.jameicaSettings.getFontData("orverw.titleFont", new FontData()));//$NON-NLS-1$
        style.setFont(font);
        return style;
    }

    /**
     * Liefert die default Sichtbarkeit der Linien.
     *
     * @return default Sichtbarkeit der Linien
     */
    public static boolean isDefaultBarVisible() {
        return Settings.jameicaSettings.getBoolean("orverw.barVisible", true); //$NON-NLS-1$
    }

    /**
     * Liefert die default Sichtbarkeit der ID.
     *
     * @return default Sichtbarkeit der ID
     */
    public static boolean isDefaultIdVisible() {
        return Settings.jameicaSettings.getBoolean("orverw.idVisible", true); //$NON-NLS-1$
    }

    /**
     * Speichert die default Linienfarbe.
     *
     * @param color
     *            the new default bar color
     */
    public static void setDefaultBarColor(final Color color) {
        Settings.jameicaSettings.setAttribute("orverw.barColor", color.getRGB()); //$NON-NLS-1$
    }

    /**
     * Setzt die Sichtbarkeit der Linien.
     *
     * @param visible
     *            the new default bar visible
     */
    public static void setDefaultBarVisible(final Boolean visible) {
        Settings.jameicaSettings.setAttribute("orverw.barVisible", visible);//$NON-NLS-1$
    }

    /**
     * Setzt den default Texttype f&uuml;r den Inhalt.
     *
     * @param style
     *            the new default content style
     */
    public static void setDefaultContentStyle(final TextStyle style) {
        Settings.jameicaSettings.setAttribute("orverw.contentAlignment", //$NON-NLS-1$
                style.getAlignment());
        Settings.jameicaSettings.setAttribute("orverw.contentColor", style //$NON-NLS-1$
                .getTextColor().getRGB());
        Settings.jameicaSettings.setAttribute("orverw.contentFont", style.getFont() //$NON-NLS-1$
                .getFontData()[0]);
    }

    /**
     * Setzt den default Texttype f&uuml;r den Fubereich.
     *
     * @param style
     *            the new default footer style
     */
    public static void setDefaultFooterStyle(final TextStyle style) {
        Settings.jameicaSettings.setAttribute("orverw.footerAlignment", style.getAlignment()); //$NON-NLS-1$
        Settings.jameicaSettings.setAttribute("orverw.footerColor", style.getTextColor().getRGB()); //$NON-NLS-1$
        Settings.jameicaSettings.setAttribute("orverw.footertFont", style.getFont().getFontData()[0]); //$NON-NLS-1$
    }

    /**
     * Setzt die default Sichtbarkeit der ID.
     *
     * @param visible
     *            the new default id visible
     */
    public static void setDefaultIdVisible(final Boolean visible) {
        Settings.jameicaSettings.setAttribute("orverw.idVisible", visible); //$NON-NLS-1$
    }

    /**
     * Setzt die default Label Hintergrundfarbe.
     *
     * @param color
     *            the new default label background
     */
    public static void setDefaultLabelBackground(final Color color) {
        Settings.jameicaSettings.setAttribute("orverw.labelBackground", color.getRGB()); //$NON-NLS-1$
    }

    /**
     * Setzt den default Texttype f&uuml;r die Register Beschreibung.
     *
     * @param style
     *            the new default register content style
     */
    public static void setDefaultRegisterContentStyle(final TextStyle style) {
        Settings.jameicaSettings.setAttribute("orverw.registerContentBackground", style //$NON-NLS-1$
                .getBackgroundColor().getRGB());
        Settings.jameicaSettings.setAttribute("orverw.registerContentColor", style //$NON-NLS-1$
                .getTextColor().getRGB());
        Settings.jameicaSettings.setAttribute("orverw.registerContentFont", style //$NON-NLS-1$
                .getFont().getFontData()[0]);
    }

    /**
     * Setzt den default Texttype f&uuml;r den Register Index.
     *
     * @param style
     *            the new default register index style
     */
    public static void setDefaultRegisterIndexStyle(final TextStyle style) {
        Settings.jameicaSettings.setAttribute("orverw.registerIndexBackground", style//$NON-NLS-1$
                .getBackgroundColor().getRGB());
        Settings.jameicaSettings.setAttribute("orverw.registerIndexColor", style//$NON-NLS-1$
                .getTextColor().getRGB());
        Settings.jameicaSettings.setAttribute("orverw.registerIndexFont", style//$NON-NLS-1$
                .getFont().getFontData()[0]);
    }

    /**
     * Setzt den default Texttype f&uuml;r den Label Titelbereich.
     *
     * @param style
     *            the new default title style
     */
    public static void setDefaultTitleStyle(final TextStyle style) {
        Settings.jameicaSettings.setAttribute("orverw.titleAlignment", //$NON-NLS-1$
                style.getAlignment());
        Settings.jameicaSettings.setAttribute("orverw.titleColor", style.getTextColor()//$NON-NLS-1$
                .getRGB());
        Settings.jameicaSettings.setAttribute("orverw.titleFont", style.getFont()//$NON-NLS-1$
                .getFontData()[0]);
    }

    /**
     * Privater Konstruktor, um ein Klassenerstellung zu verhindern.
     */
    private Settings() {
        // Intentionally left blank
    }
}
