/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.orverw.OrdnerVerwaltung;
import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.db.EmbeddedDBServiceImpl;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.messaging.QueryMessage;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.sql.version.Updater;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung unseres Datenbank Services
 *
 * @author Frank Mnster
 */
public class OrVerwDBServiceImpl extends EmbeddedDBServiceImpl implements OrVerwDBService {

    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = 8818092640375364197L;

    /**
     * Instantiates a new OrVerw DB service impl.
     *
     * @throws RemoteException
     *             rmi-exception
     */
    public OrVerwDBServiceImpl() throws RemoteException {

        super(Application.getPluginLoader().getPlugin(OrdnerVerwaltung.class).getResources().getWorkPath()
                + "/db/db.conf", "orverw", "orverwview"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        // We have to define jameicas classfinder.
        // otherwise, the db service will not be able to find
        // implementors by their interfaces.
        this.setClassFinder(Application.getClassLoader().getClassFinder());
    }

    /**
     * @see de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService#checkConsistency()
     */
    @Override
    public void checkConsistency() throws RemoteException, ApplicationException {

        Logger.info(I18N.tr("OrVerwDBServiceImpl.startUpdaterInfo")); //$NON-NLS-1$
        final UpdateProvider provider = new OrVerwUpdateProvider(this.getConnection(),
                VersionUtility.getVersion(this, "db")); //$NON-NLS-1$
        final Updater updater = new Updater(provider, "iso-8859-1"); //$NON-NLS-1$
        updater.execute();
        Logger.info(I18N.tr("OrVerwDBServiceImpl.finishUpdaterInfo")); //$NON-NLS-1$

    }

    /**
     * @see de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService#install()
     */
    @Override
    public void install() throws RemoteException {

        // Let's create an embedded Database
        final PluginResources res = Application.getPluginLoader().getPlugin(OrdnerVerwaltung.class).getResources();
        final Manifest manifest = Application.getPluginLoader().getManifest(OrdnerVerwaltung.class);
        EmbeddedDatabase embeddedDb;

        try {
            embeddedDb = new EmbeddedDatabase(res.getWorkPath() + "/db", "orverw", "orverwview"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

            // create the sql tables.
            embeddedDb.executeSQLScript(new File(manifest.getPluginDir() + File.separator + "sql", "create.sql")); //$NON-NLS-1$ //$NON-NLS-2$

        } catch (final Exception e) { // NOPMD
            throw new RemoteException(e.getMessage(), e);
        }

    }

    /**
     * @see de.willuhn.datasource.db.DBServiceImpl#getConnection()
     */
    @Override
    protected Connection getConnection() throws RemoteException {

        try {
            return super.getConnection();
        } catch (final RemoteException re) {
            // Wir benachrichtigen Jameica ueber den Fehler, damit beim Shutdown
            // kein Backup erstellt wird
            Application.getMessagingFactory().getMessagingQueue("jameica.error").sendMessage(new QueryMessage(re)); //$NON-NLS-1$
            throw re;
        }
    }

}
