/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.db.server;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.db.rmi.Version;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.util.ApplicationException;

/**
 * Utility class to get the version get the 'version' of a component from
 * database
 * 
 * @author Frank Mnster
 *
 */
public final class VersionUtility {

    /**
     * Liefert die Version der Komponenten. Ist diese noch nicht in der DB, wird
     * sie mit Version 1 angelegt.
     *
     * @param service
     *            das Datenbank Service Objekt
     * @param name
     *            Name der Komponenten deren Version geliefert werden soll.
     * @return die Version
     * @throws RemoteException
     * @throws ApplicationException
     */
    public static Version getVersion(final DBService service, final String name)
            throws RemoteException, ApplicationException {
        if (name == null || name.length() == 0) {
            throw new ApplicationException(I18N.tr("VersionUtility.noVersionNameGiven")); //$NON-NLS-1$
        }
        Version version;
        try {
            final DBIterator<Version> list = service.createList(Version.class);
            list.addFilter("name = ?", name); //$NON-NLS-1$
            if (list.hasNext()) {
                version = list.next(); // NOPMD
            } else {
                // Neue Version erstellen
                version = (Version) service.createObject(Version.class, null);
                version.setName(name);
                version.store();
            }
        } catch (final RemoteException e) {
            // Version 0 erstellen, da Versionstabelle noch nicht existiert.
            version = new VersionImpl();
            version.setName("db"); //$NON-NLS-1$
            version.setVersion(0);
        }
        return version;
    }

    private VersionUtility() {
        // Utility class must be final and have a private constructor
    }
}
