/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.dialog;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.orverw.OrdnerVerwaltung;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.LabelGroup;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;

/**
 * "AboutDialog..." Dialog.
 *
 * @author Frank Mnster
 */
public class AboutDialog extends AbstractDialog<Object> {

    /**
     * Plugin Objekt
     */
    private static final AbstractPlugin PLUGIN = Application.getPluginLoader().getPlugin(OrdnerVerwaltung.class);

    /**
     * Liefert einen modalen Dialog, der mittels .open() angezeigt werden kann
     *
     * @param position
     *            die Position, an der der Dialog angezeigt werden soll.
     *            Mgliche Werte sind POSITION_MOUSE und POSITION_CENTER.
     */
    public AboutDialog(final int position) {

	super(position);

	this.setTitle(I18N.tr("AboutDialog.about")); //$NON-NLS-1$
	this.setPanelText(I18N.tr("AboutDialog.panelText", AboutDialog.PLUGIN.getManifest().getVersion().toString())); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected Object getData() {
	return null;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) {

	final Container image = new SimpleContainer(parent, false, 2);
	final GridLayout glImage = new GridLayout();
	glImage.numColumns = 2;
	image.getComposite().setLayout(glImage);
	final CLabel imageLabel = new CLabel(image.getComposite(), SWT.NONE);
	final GridData gdImage = new GridData();
	gdImage.grabExcessHorizontalSpace = true;
	gdImage.horizontalAlignment = SWT.RIGHT;
	gdImage.horizontalSpan = 2;

	imageLabel.setImage(SWTUtil.getImage("ordner_about.png")); //$NON-NLS-1$
	imageLabel.setMargins(30, 10, 15, 30);

	final Container container = new LabelGroup(parent, I18N.tr("AboutDialog.versionInfo"), true); //$NON-NLS-1$

	final FormTextPart text = new FormTextPart();
	final StringBuilder builder = new StringBuilder(1000);

	builder.append("<form><p><b>") //$NON-NLS-1$
		.append(I18N.tr("AboutDialog.AppTitel")).append("</b></p><p>") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.licenseText")).append("<br/>[<a href=\"") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.licenseUrl")).append("\">") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.licenseUrl")).append("</a>]<br/>") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.copyrightInfo")).append(" [<a href=\"mailto:") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.mail")).append("\">") //$NON-NLS-1$ //$NON-NLS-2$
		.append(I18N.tr("AboutDialog.mail")).append("</a>]<br/><a href=\"") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.homePageUrl")).append("\">") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.homePageUrl")).append("</a><br /></p><p>") //$NON-NLS-1$//$NON-NLS-2$
		.append(I18N.tr("AboutDialog.swVersion")).append(": ") //$NON-NLS-1$//$NON-NLS-2$
		.append(AboutDialog.PLUGIN.getManifest().getVersion()).append("<br/> Build: ") //$NON-NLS-1$
		.append(AboutDialog.PLUGIN.getManifest().getBuildnumber()).append(" [") //$NON-NLS-1$
		.append(I18N.tr("AboutDialog.date")) //$NON-NLS-1$
		.append(AboutDialog.PLUGIN.getManifest().getBuildDate()).append("]</p></form>"); //$NON-NLS-1$

	text.setText(builder.toString());
	container.addPart(text);

	final ButtonArea buttons = new ButtonArea();
	buttons.addButton(I18N.tr("_close"), new Action() { //$NON-NLS-1$

	    /**
	     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	     */
	    @Override
	    public void handleAction(final Object context) throws ApplicationException {

		AboutDialog.this.close();
	    }
	}, null, true, "window-close.png"); //$NON-NLS-1$

	container.addButtonArea(buttons);

	this.setSize(360, 520);
    }

}
