/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.dialog;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.orverw.parts.labelpart.LabelSizeEnum;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 *
 */
public class NeuerOrdnerDialog extends AbstractDialog<LabelSizeEnum> {

    /**
     * Die Bezeichnung der Radio BUttun Gruppe
     */
    private static final String RADIO_GROUP_ID = "labelSize"; //$NON-NLS-1$

    /**
     * Der R&uuml;ckgabewert des Dialogfensters
     */
    private transient LabelSizeEnum result;

    /**
     * Dialog zum Auswahl der Breite eines neuen Ordners
     */
    public NeuerOrdnerDialog() {
	super(AbstractDialog.POSITION_CENTER, false);
	this.setTitle(I18N.tr("NeuerOrdnerDialog.newFolder")); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected LabelSizeEnum getData() {
	return this.result;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) throws Exception { // NOPMD

	final GridLayout layout = new GridLayout();
	layout.numColumns = 1;
	parent.setLayout(layout);

	// Erstelle einfachen Container
	final Container container = new SimpleContainer(parent, true, 1);

	RadioInput sizeRadioInput;
	for (final LabelSizeEnum labelsize : LabelSizeEnum.values()) {
	    sizeRadioInput = new RadioInput(NeuerOrdnerDialog.RADIO_GROUP_ID, true); // NOPMD
	    sizeRadioInput.setName(labelsize.getValue());
	    sizeRadioInput.setValue(labelsize);
	    sizeRadioInput.paint(container.getComposite());
	}
	RadioInput.select(NeuerOrdnerDialog.RADIO_GROUP_ID, LabelSizeEnum.getDefault());

	final ButtonArea buttons = new ButtonArea();
	buttons.addButton(I18N.tr("_create"), new Action() { //$NON-NLS-1$
	    /**
	     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	     */
	    @Override
	    public void handleAction(final Object context) throws ApplicationException {
		NeuerOrdnerDialog.this.result = (LabelSizeEnum) RadioInput.getValue(NeuerOrdnerDialog.RADIO_GROUP_ID);
		close();
	    }
	}, null, true, "ok.png"); //$NON-NLS-1$

	buttons.addButton(I18N.tr("_cancel"), new Action() { //$NON-NLS-1$
	    /**
	     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	     */
	    @Override
	    public void handleAction(final Object context) throws ApplicationException {
		NeuerOrdnerDialog.this.result = null; // NOPMD
		close();
	    }
	}, null, false, "window-close.png"); //$NON-NLS-1$

	container.addButtonArea(buttons);

    }

}
