/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.dialog;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.nebula.paperclips.core.Margins;
import org.eclipse.nebula.paperclips.core.PaperClips;
import org.eclipse.nebula.paperclips.core.PrintJob;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.print.AbstractPrint;
import de.frankmuenster.jameica.orverw.print.PrintLabel;
import de.frankmuenster.jameica.orverw.print.PrintPreviewDialog;
import de.frankmuenster.jameica.orverw.print.PrintRegister;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.frankmuenster.jameica.orverw.util.Utils;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * Creates a print dialog with options to pre-view and/or print the label and/or
 * the register.
 *
 * @author Frank Mnster
 */
public class PrintDialog extends AbstractDialog<Object> {

    /**
     * Translation key for print text
     */
    private static final String TRANS_PRINT = "_print"; //$NON-NLS-1$

    /**
     * Liste aller Ordner die angezeigt werden
     */
    private final transient List<Ordner> ordnerList;

    /**
     * Liste der Ordner mit den Print-Informationen
     */
    private transient List<PrintInfo> printInfoList;

    /**
     * @param ordnerList
     */
    public PrintDialog(final List<Ordner> ordnerList) {
        super(AbstractDialog.POSITION_CENTER, false);
        this.setTitle(I18N.tr(PrintDialog.TRANS_PRINT));
        this.setSize(450, 300);
        this.ordnerList = ordnerList;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected Object getData() throws Exception { // NOPMD

        return null;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets
     *      .Composite)
     */
    @Override
    @SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
    protected void paint(final Composite parent) throws Exception { // NOPMD

        final GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        parent.setLayout(layout);
        // Erstelle einfachen Container
        final Container containerMain = new SimpleContainer(parent, true, 1);
        // Erstelle einfachen Container
        final Container container = new SimpleContainer(containerMain.getComposite(), true, 6);
        this.printInfoList = new ArrayList<>();
        Button checkbox;
        // create labels
        this.getLabel(container, I18N.tr("PrintDialog.id")); //$NON-NLS-1$
        this.getLabel(container, I18N.tr("PrintDialog.title")); //$NON-NLS-1$
        // for the folder label
        Label label = this.getLabel(container, I18N.tr("PrintDialog.label")); //$NON-NLS-1$
        GridData griddata = new GridData();
        griddata.horizontalSpan = 2;
        griddata.horizontalAlignment = SWT.CENTER;
        label.setLayoutData(griddata);
        // for the register
        label = this.getLabel(container, I18N.tr("PrintDialog.register")); //$NON-NLS-1$
        griddata = new GridData();
        griddata.horizontalSpan = 2;
        griddata.horizontalAlignment = SWT.CENTER;
        label.setLayoutData(griddata);

        // empty line?
        label = this.getLabel(container, StringUtils.EMPTY);
        griddata = new GridData();
        griddata.horizontalSpan = 2;
        label.setLayoutData(griddata);

        // label pre-view & print
        this.getLabel(container, I18N.tr("PrintDialog.preview")); //$NON-NLS-1$
        this.getLabel(container, I18N.tr(PrintDialog.TRANS_PRINT));
        // register pre-view & print
        this.getLabel(container, I18N.tr("PrintDialog.preview")); //$NON-NLS-1$
        this.getLabel(container, I18N.tr(PrintDialog.TRANS_PRINT));

        for (final Ordner ordner : this.ordnerList) {
            final PrintInfo printInfo = new PrintInfo();
            printInfo.ordner = ordner;
            label = new Label(container.getComposite(), SWT.NONE);
            label.setText(ordner.getID());
            label = new Label(container.getComposite(), SWT.NONE);
            griddata = new GridData(); // NOPM
            griddata.grabExcessHorizontalSpace = true;
            label.setLayoutData(griddata);
            label.setText(Utils.format(ordner.getTitleText(), 20));
            checkbox = new Button(container.getComposite(), SWT.CHECK);
            griddata = new GridData(); //
            griddata.horizontalAlignment = SWT.CENTER;
            checkbox.setLayoutData(griddata);
            checkbox.setSelection(true);
            printInfo.showLabelPreView = checkbox;
            checkbox = new Button(container.getComposite(), SWT.CHECK);
            griddata = new GridData();
            griddata.horizontalAlignment = SWT.CENTER;
            checkbox.setLayoutData(griddata);
            checkbox.setSelection(false);
            printInfo.printLabel = checkbox;
            checkbox = new Button(container.getComposite(), SWT.CHECK);
            griddata = new GridData();
            griddata.horizontalAlignment = SWT.CENTER;
            checkbox.setLayoutData(griddata);
            checkbox.setSelection(true);
            printInfo.showRegPreView = checkbox;
            checkbox = new Button(container.getComposite(), SWT.CHECK);
            griddata = new GridData();
            griddata.horizontalAlignment = SWT.CENTER;
            checkbox.setLayoutData(griddata);
            checkbox.setSelection(false);
            printInfo.printRegister = checkbox;
            this.printInfoList.add(printInfo);
        }
        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(I18N.tr(PrintDialog.TRANS_PRINT), new Action() {

            @Override
            public void handleAction(final Object context) // NOPMD
                    throws ApplicationException {

                PrintDialog.this.handlePrint();
                PrintDialog.this.close();
            }
        }, null, true, "ok.png"); //$NON-NLS-1$
        buttons.addButton(I18N.tr("_cancel"), new Action() { //$NON-NLS-1$

            @Override
            public void handleAction(final Object context) // NOPMD
                    throws ApplicationException {

                PrintDialog.this.close();
            }
        }, null, false, "window-close.png"); //$NON-NLS-1$
        containerMain.addButtonArea(buttons);
    }

    /**
     * Creates a new label in the given container and with the given title
     *
     * @param container
     *            the container to add the label to
     * @param title
     *            the label title
     * @return teh new label
     */
    private Label getLabel(final Container container, final String title) {

        final Label label = new Label(container.getComposite(), SWT.NONE);
        label.setAlignment(SWT.CENTER);
        label.setText(title);
        return label;
    }

    @SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
    private void handlePrint() throws ApplicationException { // NOPMD

        for (final PrintInfo printInfo : this.printInfoList) {
            // Label
            if (printInfo.showLabelPreView.getSelection()) {
                final PrintLabel printLabel = new PrintLabel(printInfo.ordner);
                this.showPreView(printLabel);
            } else if (printInfo.printLabel.getSelection()) {
                final PrintLabel printLabel = new PrintLabel(printInfo.ordner);
                this.startPrint(printLabel);
            }
            // Register
            if (printInfo.showRegPreView.getSelection()) {
                final PrintRegister printIndex = new PrintRegister(printInfo.ordner);
                this.showPreView(printIndex);
            } else if (printInfo.printRegister.getSelection()) {
                final PrintRegister printIndex = new PrintRegister(printInfo.ordner);
                this.startPrint(printIndex);
            }
        }
    }

    private void showPreView(final AbstractPrint print) throws ApplicationException {

        final PrintPreviewDialog prevDialopg = new PrintPreviewDialog(print, this.getShell());
        try {
            prevDialopg.open();
        } catch (final Exception e) { // NOPMD
            throw new ApplicationException(I18N.tr("PrintDialog.errorShowPreview"), e); //$NON-NLS-1$
        }
    }

    private void startPrint(final AbstractPrint print) throws ApplicationException {

        final org.eclipse.swt.printing.PrintDialog dialog = new org.eclipse.swt.printing.PrintDialog(this.getShell(),
                SWT.NONE);
        final PrinterData printerData = dialog.open();
        if (printerData != null) {
            // Erstelle Printjob
            final PrintJob job = new PrintJob(I18N.tr("PrintDialog.printJobTitle") + print.getTitle(), //$NON-NLS-1$
                    print.createPrint());
            final Margins margins = job.getMargins();
            margins.left = 100;
            margins.right = 50;
            // Drucken starten
            PaperClips.print(job, printerData);
        }
    }

    /**
     * Bean Klasse mit den Printinformationen
     *
     * @author Frank Mnster
     */
    private class PrintInfo {

        /**
         * Der Ordner
         */
        private transient Ordner ordner; // NOSONAR

        /**
         * Soll der Ordnerr&uuml;cken-Preview angezeigt werden
         */
        private transient Button showLabelPreView; // NOSONAR

        /**
         * Soll der Ordnerr&uuml;cken gedruckt werden
         */
        private transient Button printLabel; // NOSONAR

        /**
         * Soll das Register angezeigt werden
         */
        private transient Button showRegPreView; // NOSONAR

        /**
         * Soll das Register gedruckt werden
         */
        private transient Button printRegister; // NOSONAR

        /**
         * Standard Konstruktor
         */
        public PrintInfo() { // NOPMD
            // empty constructor
        }
    }

}
