/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.input;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import de.willuhn.jameica.gui.input.LabelInput;

/**
 * Enhanced {@link LabelInput} to provide information if the control has been
 * created already.
 *
 * @author Frank Mnster
 * @since 1.6.0
 *
 */
public class LabelInputEnhanced extends LabelInput {

    /**
     * The label control
     */
    private Label label;
    /**
     * Background colour of the label
     */
    private Color bgColor;
    /**
     * Text alignment of the label test
     */
    private Integer textAlignment;
    /**
     * Visibility of label
     */
    private boolean visible;
    /**
     * Width of the label
     */
    private final int width;

    /**
     * Creates a new LabelInput with the given text
     *
     * @param text
     *            The text to display
     */
    public LabelInputEnhanced(final String text, final int width) {
        super(text);
        // save width
        this.width = width;
    }

    @Override
    public Control getControl() {

        // Set new layout to parent to avoid grabbing horizontal space
        final GridData gridData = new GridData();
        gridData.widthHint = this.width;
        getParent().setLayoutData(gridData);

        // get the label
        this.label = (Label) super.getControl();

        // set text alignment
        if (this.textAlignment != null) {
            this.label.setAlignment(this.textAlignment);
        }
        // set the background colour
        if (this.bgColor != null) {
            this.label.setBackground(this.bgColor);
        }

        // return the label
        return this.label;

    }

    /**
     * Gives back <i>true</i> in case the parent composite is not null and is
     * not disposed.
     *
     * @return
     */
    public boolean hasControl() {

        return this.label == null ? false : true;
    }

    /**
     * Returns <code>true</code> if the receiver is visible and all ancestors up
     * to and including the receiver's nearest ancestor shell are visible.
     * Otherwise, false is returned.
     *
     * @return the receiver's visibility state
     */
    public boolean isVisible() {
        return this.visible;
    }

    /**
     * Set the backgorund colour
     *
     * @param color
     *            background colour.
     */
    public void setBackground(final Color color) {
        this.bgColor = color;
        if (this.label != null && !this.label.isDisposed()) {
            this.label.setBackground(this.bgColor);
        }
    }

    /**
     * Controls how text and images will be displayed in the receiver. The
     * argument should be one of <code>LEFT</code>, <code>RIGHT</code> or
     * <code>CENTER</code>.
     *
     * @param alignment
     *            the new alignment
     */
    public void setTextAlignment(final int alignment) {
        this.textAlignment = alignment;
        if (this.label != null && !this.label.isDisposed()) {
            this.label.setAlignment(this.textAlignment);
        }
    }

    /**
     * Marks the receiver as visible if the argument is <code>true</code>, and
     * marks it invisible otherwise. <br/>
     * <br />
     * If one of the receiver's ancestors is not visible or some other condition
     * makes the receiver not visible, marking it visible may not actually cause
     * it to be displayed
     *
     * @param visible
     *            the new visibility state
     */
    public void setVisible(final boolean visible) {
        this.visible = visible;
        if (this.label != null && !this.label.isDisposed()) {
            this.label.setVisible(this.visible);
        }

    }

}
