/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.input;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.parts.Button;

/**
 * Erweitert den Button um einen Tooltiptext
 *
 * @author Frank Mnster
 *
 */
public class ToolTipButton extends Button {

    /**
     * Tool-Tip-Text
     */
    private String toolTip = StringUtils.EMPTY;

    /**
     *
     * @param title
     *            Beschriftung.
     * @param action
     *            Action, die beim Klick ausgefuehrt werden soll.
     */
    public ToolTipButton(final String title, final Action action) {
	this(title, action, null, false);
    }

    /**
     *
     * @param title
     *            Beschriftung.
     * @param action
     *            Action, die beim Klick ausgefuehrt werden soll.
     * @param context
     *            ein Context-Objekt, welches beim Click der Action uebergeben
     *            wird.
     */
    public ToolTipButton(final String title, final Action action, final Object context) {
	this(title, action, context, false);
    }

    /**
     *
     * @param title
     *            Beschriftung.
     * @param action
     *            Action, die beim Klick ausgefuehrt werden soll.
     * @param context
     *            ein Context-Objekt, welches beim Click der Action uebergeben
     *            wird.
     * @param defaultButton
     *            legt fest, ob das der Default-Button der Shell sein soll.
     */
    public ToolTipButton(final String title, final Action action, final Object context, final boolean defaultButton) {
	this(title, action, context, defaultButton, null);
    }

    /**
     * @param title
     *            Beschriftung.
     * @param action
     *            Action, die beim Klick ausgefuehrt werden soll.
     * @param context
     *            ein Context-Objekt, welches beim Click der Action uebergeben
     *            wird.
     * @param defaultButton
     *            legt fest, ob das der Default-Button der Shell sein soll.
     * @param icon
     *            Icon, welches links neben der Beschriftung angezeigt werden
     *            soll.
     */
    public ToolTipButton(final String title, final Action action, final Object context, final boolean defaultButton,
	    final String icon) {
	super(title, action, context, defaultButton, icon);
    }

    /**
     * @return toolTip
     */
    public String getToolTip() {
	return this.toolTip;
    }

    /**
     * @see de.willuhn.jameica.gui.parts.Button#paint
     *      (org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {
	super.paint(parent);
	this.button.setToolTipText(this.toolTip);
    }

    /**
     * @param toolTip
     *            das zu setzende Objekt toolTip
     */
    public void setToolTip(final String toolTip) {
	this.toolTip = toolTip;
    }

}
