/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.navigation;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.GenericObject;
import de.willuhn.datasource.GenericObjectNode;
import de.willuhn.datasource.pseudo.PseudoIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Item;
import de.willuhn.jameica.gui.NavigationItem;
import de.willuhn.jameica.gui.util.SWTUtil;

/**
 * AktenVerwaltung Navigation Items
 *
 * @author Frank Mnster
 */
public class OrdnerVerwaltungItem implements NavigationItem {

    /**
     * Das Eltern-Element des Navigationsbaumes
     */
    private transient NavigationItem parent;

    /**
     * Die 'Action' die mit dem Men&uuml;punkt ausgef&uuml;hrt werden soll
     */
    private transient Action action;

    /**
     * Der Text den Men&uuml;punktes
     */
    private transient String navitext;

    /**
     * Ein Liste mit allen Kind-Elementen
     */
    private transient List<Item> children;

    /**
     * Das Icon des Men&uuml;punktes
     */
    private transient String icon;

    /**
     * Ist der Men&uuml;punkt aktiviert
     */
    private transient Boolean enabled;

    /**
     * Erstellt ein AktenVerwaltung Navigation Item.<br/>
     * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i> initialisiert.
     *
     * @param item
     *            ist das Elternelement des Navigationsitems
     * @param navitext
     *            der Text, der angezeigt wird
     * @param action
     *            die Klasse, die das Interface <b>Action</b> implementiert und durch das NavigationItem ausgefhrt
     *            werden soll.
     */
    public OrdnerVerwaltungItem(final NavigationItem item, final String navitext, final Action action) {

        this(item, navitext, action, null);
    }

    /**
     * Erstellt ein AktienKonto Navigation Item.<br/>
     * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i> initialisiert.
     *
     * @param item
     *            ist das Elternelement des Navigationsitems
     * @param navitext
     *            der Text, der angezeigt wird
     * @param action
     *            die Klasse, die das Interface <b>Action</b> implementiert und durch das NavigationItem ausgefhrt
     *            werden soll.
     * @param icon
     *            String des Icon-Bildes des NavigationItems
     */
    public OrdnerVerwaltungItem(final NavigationItem item, final String navitext, final Action action,
            final String icon) {

        this(item, navitext, action, icon, true);
    }

    /**
     * Erstellt ein AktienKonto Navigation Item.<br/>
     * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i> initialisiert.
     *
     * @param item
     *            ist das Elternelement des Navigationsitems
     * @param navitext
     *            der Text, der angezeigt wird
     * @param action
     *            die Klasse, die das Interface <b>Action</b> implementiert und durch das NavigationItem ausgefhrt
     *            werden soll.
     * @param icon
     *            String des Icon-Bildes des NavigationItems
     * @param enabled
     *            setzt den Aktiv-Status des Items
     */
    public OrdnerVerwaltungItem(final NavigationItem item, final String navitext, final Action action,
            final String icon, final Boolean enabled) {

        this.parent = item;
        this.action = action;
        this.navitext = navitext;
        this.icon = icon;
        this.enabled = enabled;
        this.children = new ArrayList<>();
    }

    /**
     * @see NavigationItem#addChild(Item)
     */
    @Override
    public void addChild(final Item item) throws RemoteException {

        throw new RemoteException(I18N.tr("OrdnerVerwaltungItem.noChildrenAllowed")); //$NON-NLS-1$
    }

    /**
     * @see GenericObject#equals(GenericObject)
     */
    @Override
    @SuppressWarnings("PMD.SuspiciousEqualsMethodName")
    public boolean equals(final GenericObject arg0) throws RemoteException { // NOSONAR

        return arg0 instanceof OrdnerVerwaltungItem && this.getID().equals(arg0.getID());
    }

    /**
     * @see Item#getAction()
     */
    @Override
    public Action getAction() throws RemoteException {

        return this.action;
    }

    /**
     * @see GenericObject#getAttribute(String)
     */
    @Override
    public Object getAttribute(final String arg0) throws RemoteException {

        return this.getName();
    }

    /**
     * @see GenericObject#getAttributeNames()
     */
    @Override
    public String[] getAttributeNames() throws RemoteException {

        return new String[] { "name" }; //$NON-NLS-1$
    }

    /**
     * @see GenericObjectNode#getChildren()
     */
    @Override
    public GenericIterator<?> getChildren() throws RemoteException {

        return PseudoIterator.fromArray(this.children.toArray(new OrdnerVerwaltungItem[this.children.size()]));
    }

    /**
     * @see de.willuhn.jameica.gui.extension.Extendable#getExtendableID()
     */
    @Override
    public String getExtendableID() {

        return this.getID();
    }

    /**
     * @see NavigationItem#getIconClose()
     */
    @Override
    public Image getIconClose() throws RemoteException {

        return SWTUtil.getImage(this.icon == null ? "ordner.png" : this.icon); //$NON-NLS-1$
    }

    /**
     * @see NavigationItem#getIconOpen()
     */
    @Override
    public Image getIconOpen() throws RemoteException {

        return SWTUtil.getImage(this.icon == null ? "ordner.png" : this.icon); //$NON-NLS-1$
    }

    /**
     * @see GenericObject#getID()
     */
    @Override
    public String getID() {

        return this.getClass().getName() + "." + this.getName(); //$NON-NLS-1$
    }

    /**
     * @see Item#getName()
     */
    @Override
    public String getName() {

        return this.navitext;
    }

    /**
     * @see GenericObjectNode#getParent()
     */
    @Override
    public GenericObjectNode getParent() throws RemoteException {

        return this.parent;
    }

    /**
     * @see GenericObjectNode#getPath()
     */
    @Override
    public GenericIterator<?> getPath() throws RemoteException {

        @SuppressWarnings("unchecked")
        final List<NavigationItem> list = PseudoIterator.asList(this.parent.getPath());
        list.add(this);
        return PseudoIterator.fromArray(list.toArray(new NavigationItem[0]));
    }

    /**
     * @see GenericObjectNode#getPossibleParents()
     */
    @Override
    public GenericIterator<?> getPossibleParents() throws RemoteException {

        throw new RemoteException("not implemented"); //$NON-NLS-1$
    }

    /**
     * @see GenericObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {

        return "name"; //$NON-NLS-1$
    }

    /**
     * @see GenericObjectNode#hasChild(GenericObjectNode)
     */
    @Override
    public boolean hasChild(final GenericObjectNode arg0) throws RemoteException {

        return false;
    }

    /**
     * @see Item#isEnabled()
     */
    @Override
    public boolean isEnabled() throws RemoteException {

        return this.enabled;
    }

    /**
     * @see NavigationItem#isExpanded()
     */
    @Override
    public boolean isExpanded() throws RemoteException {

        return false;
    }

    /**
     * @see Item#setEnabled(boolean, boolean)
     */
    @Override
    public void setEnabled(final boolean enabled, final boolean recursive) throws RemoteException {

        this.enabled = enabled;
    }
}
