/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.gui.views;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Layout;

import de.frankmuenster.jameica.orverw.gui.controller.OrdnerListViewController;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.View;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * StartView von Ordner Verwaltung.
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class OrdnerListView extends AbstractView {

    /**
     * Unser Standartkonstruktor
     */
    public OrdnerListView() {
	super();
    }

    /**
     * this method will be invoked when starting the view.
     * 
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws RemoteException {

	final View view = GUI.getView();
	final OrdnerListViewController controller = new OrdnerListViewController(this);

	view.setTitle(I18N.tr("OrdnerListView.title")); //$NON-NLS-1$

	final Container container = new SimpleContainer(getParent());

	// layout with 2 columns
	final ColumnLayout columns = new ColumnLayout(container.getComposite(), 2);
	final Layout parentLayout = columns.getComposite().getLayout();
	if (parentLayout instanceof GridLayout) {
	    ((GridLayout) parentLayout).makeColumnsEqualWidth = false;
	}

	// left side
	final Container left = new SimpleContainer(columns.getComposite(), true);
	final ButtonArea buttonArea = new ButtonArea();

	buttonArea.addButton(controller.getNeuerOrdner());
	buttonArea.addButton(controller.getSettings());
	left.addPart(buttonArea);
	left.addPart(controller.getOrdnerTabelle());

	// right side
	final Container right = new SimpleContainer(columns.getComposite(), true, 1);
	final GridData gdRight = new GridData();
	gdRight.horizontalAlignment = SWT.CENTER;
	gdRight.widthHint = 180;
	right.getComposite().setLayoutData(gdRight);

	final CLabel imageLabel = new CLabel(right.getComposite(), SWT.NONE);
	imageLabel.setImage(SWTUtil.getImage("ordner_start_view.png")); //$NON-NLS-1$
	imageLabel.setMargins(15, 10, 15, 10);

    }

    /**
     * this method will be executed when exiting the view. You don't need to
     * dispose your widgets, the GUI controller will do this in a recursive way
     * for you.
     * 
     * @see de.willuhn.jameica.gui.AbstractView#unbind()
     */
    @Override
    public void unbind() throws ApplicationException {
	// We've nothing to do here ;)
    }

}
