/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.parts;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.frankmuenster.jameica.orverw.util.Utils;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.TablePart;

/**
 * Part das alle gespeicherten Ordner in einer Tabelle anzeigt.
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class OrdnerTablePart extends TablePart implements Part {

    /**
     * Liefert eine Liste mit alle Ordner in der Datenbank, sortier nach ID.
     *
     * @return eine GenericIterator-Object mit alle Ordner
     * @throws RemoteException
     */
    private static GenericIterator<Ordner> init() throws RemoteException {
	final OrVerwDBService service = (OrVerwDBService) Settings.getDBService();
	final DBIterator<Ordner> list = service.createList(Ordner.class);
	list.setOrder("ORDER BY id asc"); //$NON-NLS-1$
	return list;
    }

    /**
     * Konstruktor zum Erstellen der Tabelle mit allen Ordnern.
     *
     * @param action
     *            Aktion beim Doppelklick
     * @throws RemoteException
     */
    public OrdnerTablePart(final Action action) throws RemoteException {
	this(OrdnerTablePart.init(), action);
    }

    /**
     * Konstruktor zum Erstellen der Tabelle mit &uuml;bergebener Ordner Liste.
     *
     * @param list
     *            Liste der Ordner die angezeigt werden soll
     * @param action
     *            Aktion beim Doppelklick
     */
    public OrdnerTablePart(final GenericIterator<Ordner> list, final Action action) {
	super(list, action);

	setRememberOrder(true);
	setRememberState(true);
	setMulti(true);

	addColumn(I18N.tr("OrdnerTablePart.columnId"), "id", null, false); //$NON-NLS-1$ //$NON-NLS-2$
	addColumn(I18N.tr("OrdnerTablePart.columnTitel"), "title", new LabelContent(20), false); //$NON-NLS-1$ //$NON-NLS-2$
	addColumn(I18N.tr("OrdnerTablePart.columnContent"), "content", new LabelContent(40), false); //$NON-NLS-1$ //$NON-NLS-2$

    }

    /**
     * Ein Formatter zum bearbeiten der Titeltexte. Zeilenumbr&uuml;che werden
     * durch " \ " ersetzt, damit es in der Tabelle sch&ouml;ner aussieht.
     *
     * @author Frank Mnster
     *
     */
    private class LabelContent implements Formatter {

	/**
	 * Maximale L&auml;nge des Textes
	 */
	private final transient int length;

	/**
	 * Konstruktoer zum erstellen des Formatters mit angabe der maximalen
	 * L&auml;nge des Textes.
	 *
	 * @param length
	 *            die maximale L&auml;nge des Textes. Ist der Text
	 *            l&auml;nger, wird der Rest durch "..." ersetzt
	 */
	public LabelContent(final int length) {
	    this.length = length;

	}

	/**
	 * @see de.willuhn.jameica.gui.formatter.Formatter#format(java.lang.Object)
	 */
	@Override
	public String format(final Object object) {

	    String result = StringUtils.EMPTY;

	    if (object instanceof String) {
		result = Utils.format((String) object, this.length);
	    }
	    return result;

	}

    }

}
