/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.parts;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.TablePart;

/**
 * Part das eine Tabelle mit allen Registereintr%auml;gen eines Ordner anzeigt.
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class RegisterTablePart extends TablePart implements Part {

    /**
     * Liefert eine Liste mit allen Registereintr&auml;gen in der Datenbank
     *
     * @return GenericIterator-Object mit allen Registereintr&auml;gen
     * @throws RemoteException
     */
    private static GenericIterator<Register> init() throws RemoteException {
	final OrVerwDBService service = (OrVerwDBService) Settings.getDBService();
	final DBIterator<Register> list = service.createList(Register.class);
	list.setOrder("ORDER BY id asc"); //$NON-NLS-1$

	return list;
    }

    /**
     * Konstruktor zum Erstellen der Tabelle mit allen Regsitereintr&auml;gen in
     * der Datenbank. Sollte eigentlich nide benutzt werden.
     *
     * @param action
     *            Aktion bei Doppelklick
     * @throws RemoteException
     */
    public RegisterTablePart(final Action action) throws RemoteException {
	this(RegisterTablePart.init(), action);
    }

    /**
     * Konstruktor zum Erstellen der Tabelle mit &Uuml;bergabe der Liste.
     *
     * @param list
     *            Liste mit Registereintr&auml;gen
     * @param action
     *            Aktion bei Doppelklick
     */
    public RegisterTablePart(final GenericIterator<Register> list, final Action action) {
	super(list, action);

	setRememberColWidths(true);
	setRememberOrder(true);
	setRememberState(true);
	setEnabled(true);

	// Wichtig Tabellenfelder angeben, nicht die aus der korespondierenden
	// Klasse !!!
	addColumn(I18N.tr("RegisterTablePart.index"), "idx", null, true); //$NON-NLS-1$ //$NON-NLS-2$
	addColumn(I18N.tr("RegisterTablePart.description"), "content", null, true); //$NON-NLS-1$ //$NON-NLS-2$

    }

}
