/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.parts.labelpart;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Caret;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelButtonListener;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelInputButtons;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.frankmuenster.jameica.orverw.util.Utils;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;

/**
 * Ein Texteingabefeld bei dem die Gr&ouml;&szlig;e, der Schrifttyp und Farben
 * angegeben werden k&ouml;nnen. Dazu gibt es noch einen Listener, der
 * benachrichtigt, wenn der Text ver&auml;ndert wurde.
 *
 * @author Frank Mnster
 * @since 1.0
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.TooManyMethods" })
public class LabelInputText implements Part {

    /**
     * Identifier for editor button
     */
    private static final String EDITOR_BUTTON_ID = "BUTTON"; //$NON-NLS-1$

    /**
     * Translation key for LabelInputText.doubleClickToEdit
     */
    private static final String I18N_DOUBLE_CLICK_TO_EDIT = "LabelInputText.doubleClickToEdit"; //$NON-NLS-1$

    /**
     * Shell the text be vertically centred
     */
    private boolean centerVertical;

    /**
     * The left and right margin in case text it not centred Always 4 mm
     */
    private final int leftRightMargin = Utils.mmToDisplayPoints(LabelPart.TEXT_LEFT_RIGHT_MARGIN);

    /**
     * Der Inhalt des Eingabefeldes. Wird bentigt um nderungen zu sehen.
     */
    private String text;

    /**
     * Die Gre des Eingabefeldes
     */
    private final Point size;

    /**
     * Der Schrifttyp des Textes
     */
    private Font font;

    /**
     * Die Textfarbe
     */
    private Color foreground;

    /**
     * Die Hintergrundfarbe
     */
    private Color background;

    /**
     * Die Textausrichtung
     */
    private int textAlignment;

    /**
     * Die Listener fr einen Button-Click-Event
     */
    private final transient Set<LabelInputTextChangeListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * The direction of the text. Can be SWT.LEFT, SWT.RIGHT or SWT.HORIZONTAL
     */
    private int textDirection;

    /**
     * Shell for pop-up editor
     */
    private Shell eShell;
    /**
     * Text widget for editor
     */
    private StyledText eInputText;

    /**
     * Label to paint text on
     */
    private Label label;

    /**
     * Creates a new LabelInputText with the size and intent
     *
     * @param size
     *            the initial size
     *
     */
    public LabelInputText(final Point size) {

        this(size, SWT.LEFT);

    }

    /**
     * Creates a new LabelInputText with the size and text alignment.
     *
     * @param size
     *            the initial size
     *
     * @param textAlignment
     *            the initial text alignment
     */
    public LabelInputText(final Point size, final int textAlignment) {

        this(size, textAlignment, SWT.HORIZONTAL);
    }

    /**
     * Creates a new LabelInputText with the size and text alignment.
     *
     * @param size
     *            the initial size
     * @param textAlignment
     *            the initial text alignment
     * @param textDirection
     *            he direction of the text. Can be SWT.LEFT, SWT.RIGHT or
     *            SWT.HORIZONTAL
     *
     */
    public LabelInputText(final Point size, final int textAlignment, final int textDirection) {

        this.size = size;
        this.textAlignment = textAlignment;
        this.textDirection = textDirection;
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final LabelInputTextChangeListener listener) {

        this.listeners.add(listener);
    }

    /**
     * @return background
     */
    public Color getBackground() {

        return this.eInputText.getBackground();
    }

    /**
     * @return font
     */
    public Font getFont() {

        return this.eInputText.getFont();
    }

    /**
     * @return fontColor
     */
    public Color getForeground() {

        return this.eInputText.getForeground();
    }

    /**
     * @return size
     */
    public Point getSize() {

        return this.size;
    }

    /**
     * @return text
     */
    public String getText() {

        return this.text;
    }

    /**
     * @return textAlignment
     */
    public int getTextAlignment() {

        return this.textAlignment;
    }

    /**
     * @return textDirection
     */
    public int getTextDirection() {

        return this.textDirection;
    }

    public boolean isCenterVertical() {
        return this.centerVertical;
    }

    /**
     * Notifies registered listeners, that the text has changed
     */
    public final void notifyListeners() {

        for (final LabelInputTextChangeListener listener : this.listeners) {
            listener.notifyTextChanged();
        }
    }

    /**
     * @param parent
     */
    @Override
    public void paint(final Composite parent) {

        if (this.label != null) {
            return;
        }

        // Create and size label
        this.label = new Label(parent, SWT.NONE);
        final GridData gridData = new GridData();
        gridData.heightHint = this.size.y;
        gridData.widthHint = this.size.x;
        gridData.horizontalIndent = 0;
        gridData.verticalIndent = 0;
        this.label.setLayoutData(gridData);
        this.label.setToolTipText(I18N.tr(LabelInputText.I18N_DOUBLE_CLICK_TO_EDIT));
        // attach the dispose and mouse listener
        addLabelListeners();

        // create the editor
        createEditor();

        // finally draw what we have already
        this.label.redraw();
        this.label.update();
        this.eShell.setVisible(true);

        draw();
        this.eShell.setVisible(false);
    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final LabelInputTextChangeListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * @param background
     *            das zu setzende Objekt background
     */
    public void setBackground(final Color background) {

        if (background != null) {
            this.background = background;
            if (this.eInputText != null) {
                this.eInputText.setBackground(background);
                updateLabel();
            }
        }
    }

    public void setCenterVertical(final boolean centerVertical) {
        this.centerVertical = centerVertical;
    }

    /**
     * @param font
     *            das zu setzende Objekt font
     */
    public void setFont(final Font font) {

        if (font != null) {
            this.font = font;
            if (this.eInputText != null) {
                this.eInputText.setFont(font);
                updateLabel();
            }
        }
    }

    /**
     * @param color
     *            das zu setzende Objekt fontColor
     */
    public void setForeground(final Color color) {

        if (color != null) {
            this.foreground = color;
            if (this.eInputText != null) {
                this.eInputText.setForeground(color);
                updateLabel();
            }
        }
    }

    /**
     * Set the text to display and redraw the label if not NULL.
     *
     * @param text
     *            das zu setzende Objekt text
     */
    public void setText(final String text) {

        if (text != null) {
            this.text = text;
            if (this.label != null) {
                this.eInputText.setText(text);
                updateLabel();
            }
        }
    }

    /**
     * @param textAlignment
     *            die zu setzende Textausrichtung
     */
    public void setTextAlignment(final int textAlignment) {

        this.textAlignment = textAlignment;
        if (this.eInputText != null) {
            this.eInputText.setAlignment(textAlignment);
            updateLabel();
        }
    }

    /**
     * @param textDirection
     *            set text direction to set. Can be SWT.LEFT, SWT.RIGHT or
     *            SWT.HORIZONTAL
     */
    public void setTextDirection(final int textDirection) {

        this.textDirection = textDirection;

        // re-draw the label if eShell exists
        if (this.eShell != null) {
            // update the label
            updateLabel();
        }
    }

    /**
     * Add the save and cancel buttons to the editor
     */
    private void addButtonsToEditor() {

        // Editor button mouse listener
        final MyEditorButtonMouseListener editorBtnMouseListener = new MyEditorButtonMouseListener();

        // Save button as label
        final Label save = new Label(this.eShell, SWT.SHADOW_OUT);
        final FontData fontdata = save.getFont().getFontData()[0];
        fontdata.setHeight(8);
        final Font myFont = new Font(save.getFont().getDevice(), fontdata);
        save.setFont(myFont);
        save.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        save.setText(I18N.tr("_save")); //$NON-NLS-1$
        save.setData(LabelInputText.EDITOR_BUTTON_ID, "SAVE"); //$NON-NLS-1$
        // add the listener
        save.addMouseListener(editorBtnMouseListener);

        // empty label in the middle
        new Label(this.eShell, SWT.SHADOW_OUT);

        // cancel button as label
        final Label cancel = new Label(this.eShell, SWT.SHADOW_OUT);
        cancel.setFont(myFont);
        cancel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, true));
        cancel.setText(I18N.tr("_cancel")); //$NON-NLS-1$
        // add the listener
        cancel.addMouseListener(editorBtnMouseListener);

    }

    /**
     * Adds a {@link ModifyListener} to the eInputText widget to act in case the
     * text height changes.
     */
    private void addEditorListeners() {

        // add text change listener
        this.eInputText.addModifyListener(new ModifyListener() {

            @Override
            public void modifyText(final ModifyEvent event) {
                // re-calculate margins and size
                setTextInputSizeAndMargins();
            }
        });
    }

    /**
     * Adds a {@link DisposeListener} and a {@link MouseListener to the label
     * widget to act in case the text height changes.
     */
    private void addLabelListeners() {

        // add a dispose listener to dispose the Shell and StyledText used for
        // text input
        this.label.addDisposeListener(new DisposeListener() {

            @Override
            public void widgetDisposed(final DisposeEvent event) {

                if (LabelInputText.this.eInputText != null && !LabelInputText.this.eInputText.isDisposed()) {
                    LabelInputText.this.eInputText.dispose();
                }
                if (LabelInputText.this.eShell != null && !LabelInputText.this.eShell.isDisposed()) {
                    LabelInputText.this.eShell.dispose();
                }
            }
        });

        // Adds the mouse listeners to display editor respectively to save the
        // text and re-draw the label.
        this.label.addMouseListener(new MouseListener() {

            @Override
            public void mouseDoubleClick(final MouseEvent event) {

                Logger.debug(String.format("Event as String %s", event.getSource().toString())); //$NON-NLS-1$

                // Runnable to create the editor shell and input field
                final Runnable runnable = () -> {
                    if (LabelInputText.this.eShell == null) {
                        // can normally not happen
                        Logger.error("Erstellen der Texteingabefelder ist fehlgeschlagen. Bitte den Fehler melden"); //$NON-NLS-1$
                    } else if (LabelInputText.this.eShell.isVisible()) {
                        // Save the text and hide the editor
                        saveTextAndHideEditor();
                    } else {
                        // set the initial text
                        LabelInputText.this.eInputText.setText(LabelInputText.this.text);

                        // Set margins and re-layout the editor
                        setTextInputSizeAndMargins();

                        // Calculate position
                        final Point location = calculateEditorPosition();
                        LabelInputText.this.eShell.setLocation(location);

                        // editor hidden, make visible
                        LabelInputText.this.eShell.setVisible(true);
                        // change tool tip text
                        LabelInputText.this.label.setToolTipText(I18N.tr("LabelInputText.doubleClickToSave")); //$NON-NLS-1$
                    }
                };

                // open the editor
                GUI.getDisplay().syncExec(runnable);
            }

            @Override
            public void mouseDown(final MouseEvent event) {
                // Nothing to do here
            }

            @Override
            public void mouseUp(final MouseEvent event) {
                // Nothing to do here

            }
        });
    }

    /**
     * Calculates the position to display the editor 'window'
     *
     * @return a Point with the x and y coordinates
     */
    private Point calculateEditorPosition() {

        // Set label bounds
        final Rectangle bounds = this.label.getBounds();
        // calculate position
        final Point calculatedPosition = this.label.toDisplay(bounds.width / 2, 10);

        // check if the whole editor is visible
        // first get the bounds of the editor (eShell) and the monitors
        final Rectangle eShellBounds = this.eShell.getBounds();
        final Monitor[] monitors = this.eShell.getDisplay().getMonitors();

        // now find the monitor we are on
        for (int i = 0; i < monitors.length; i++) {
            if (monitors[i].getBounds().intersects(eShellBounds)) {

                // get monitor size
                final Rectangle monitorBounds = monitors[i].getBounds();
                // calculate the maximal x position
                final int maxX = monitorBounds.width - eShellBounds.width - 20;
                // set x position regarding to the minimum
                calculatedPosition.x = Math.min(maxX, calculatedPosition.x);
                // calculate the maximal y position
                final int maxY = monitorBounds.height - eShellBounds.height - 20;
                // set y position regarding to the minimum
                calculatedPosition.y = Math.min(maxY, calculatedPosition.y);

            }
        }
        // return the calculated position
        return calculatedPosition;

    }

    private int centerText(final int fieldHeight) {

        int textHeight = 0;
        if (this.eInputText.getText().length() > 0) {
            textHeight = this.eInputText.getTextBounds(0, this.eInputText.getText().length() - 1).height;
        }

        // in case last line is empty --> add one line height
        if (StringUtils.isEmpty(this.eInputText.getLine(this.eInputText.getLineCount() - 1))) {
            textHeight += this.eInputText.getLineHeight();
        }

        // calculate the margin, cannot be negative
        return Math.max((fieldHeight - textHeight) / 2, 0);

    }

    /**
     * Creates the editor in a new {@link Shell}
     */
    private void createEditor() {
        // create new shell: non-modal
        this.eShell = new Shell(GUI.getDisplay().getActiveShell(), SWT.NO_TRIM | SWT.PRIMARY_MODAL);

        final GridLayout gridLayout = new GridLayout(3, false);
        gridLayout.marginWidth = 5;
        gridLayout.marginHeight = 5;
        gridLayout.verticalSpacing = 5;
        this.eShell.setLayout(gridLayout);

        // new Text input field
        this.eInputText = new StyledText(this.eShell, SWT.BORDER);
        this.eInputText.setLayoutData(new GridData());

        this.eInputText.setWordWrap(true);
        this.eInputText.setAlignment(getTextAlignment());

        if (this.text != null) {
            this.eInputText.setText(this.text);
        }
        if (this.font != null) {
            this.eInputText.setFont(this.font);
        }
        if (this.foreground != null) {
            this.eInputText.setForeground(this.foreground);
        }
        if (this.background != null) {
            this.eInputText.setBackground(this.background);
        }

        // Composite for the font style buttons
        final Composite buttons = new Composite(this.eShell, SWT.BORDER);
        new LabelInputButtons(buttons, null, new MyButtonLister());

        // add save and cancel buttons
        addButtonsToEditor();

        // add Editor listeners
        addEditorListeners();

        // Set the TextInput size and margins
        setTextInputSizeAndMargins();

        // now finally pack to have all correctly set
        this.eShell.pack();

        // hide the text box
        this.eShell.setVisible(false);

        // open the shell
        this.eShell.open();

    }

    /**
     * Here the content of the {@link StyledText} is draw as image with the
     * correct rotation.
     *
     * @param force
     *
     */
    private void draw() {

        // only if eText
        if (this.eInputText != null && this.label != null) {

            final boolean wasVisible = this.eShell.isVisible();

            if (!this.eShell.isVisible()) {

                // Set margins and re-layout the editor
                setTextInputSizeAndMargins();

                // Calculate position
                final Point location = calculateEditorPosition();
                this.eShell.setLocation(location);

                // Show shell
                this.eShell.setVisible(true);
            }

            // get the caret to hide it
            final Caret caret = this.eInputText.getCaret();
            caret.setVisible(false);

            // redraw the editor to be sure all is painted
            this.eInputText.requestLayout();
            this.eInputText.redraw();
            this.eInputText.update();

            // calculate image size
            Logger.debug(String.format("Border size: %d", this.eInputText.getBorderWidth())); //$NON-NLS-1$
            final Point tableSize = this.eInputText.getSize();
            Logger.debug("eInput original size: " + tableSize); //$NON-NLS-1$
            tableSize.x -= this.eInputText.getBorderWidth() * 2;
            tableSize.y -= this.eInputText.getBorderWidth() * 2;
            Logger.debug("eInput fixed size: " + tableSize); //$NON-NLS-1$
            // create the image
            Image image = new Image(GUI.getDisplay(), tableSize.x, tableSize.y);

            // copy from text input
            final GC gc = new GC(this.eInputText); // NOPMD
            gc.copyArea(image, 0, 0);
            gc.dispose();

            // set caret visible again
            caret.setVisible(true);

            // reset shell display to original setting
            this.eShell.setVisible(wasVisible);

            // Text to be displayed vertical
            if (this.textDirection != SWT.HORIZONTAL) {
                // rotate image
                final Image nImage = rotateImage(image, this.textDirection);
                image.dispose();
                image = nImage;
            }

            // copy the image in the correct rotation on the label
            this.label.setImage(image);
        }
    }

    /**
     * Rotates an image by 90 degrees to the left or right
     *
     * @param image
     *            the e to be rotated
     * @param direction
     *            can be SWT.LEFT or SWT.RIGHT
     */

    private Image rotateImage(final Image image, final int direction) {

        final ImageData iData = image.getImageData();

        final int bytesPerPixel = iData.bytesPerLine / iData.width;
        final int destBytesPerLine = iData.height * bytesPerPixel;
        final byte[] newData = new byte[iData.width * destBytesPerLine];
        int width = 0;
        int height = 0;
        for (int srcY = 0; srcY < iData.height; srcY++) {
            for (int srcX = 0; srcX < iData.width; srcX++) {
                int destX = 0;
                int destY = 0;
                int destIndex = 0;
                int srcIndex = 0;
                if (direction == SWT.LEFT) {
                    // left 90 degrees
                    destX = srcY;
                    destY = iData.width - srcX - 1;
                    width = iData.height;
                    height = iData.width;
                } else {
                    // right 90 degrees
                    destX = iData.height - srcY - 1;
                    destY = srcX;
                    width = iData.height;
                    height = iData.width;
                }
                destIndex = destY * destBytesPerLine + destX * bytesPerPixel;
                srcIndex = srcY * iData.bytesPerLine + srcX * bytesPerPixel;
                System.arraycopy(iData.data, srcIndex, newData, destIndex, bytesPerPixel);
            }
        }
        // destBytesPerLine is used as scanlinePad to ensure that no padding is
        // required
        final ImageData iNewData = new ImageData(width, height, iData.depth, iData.palette, destBytesPerLine, newData);
        return new Image(image.getDevice(), iNewData);
    }

    /**
     * Save the text from the editor, redraws the label, hides the editor and
     * notifies listeners, that the text has changed.
     */
    private void saveTextAndHideEditor() {

        // save text, text style and colour
        this.font = this.eInputText.getFont();
        this.foreground = this.eInputText.getForeground();
        this.textAlignment = this.eInputText.getAlignment();
        this.text = this.eInputText.getText();
        // redraw the label
        draw();
        // hide editor
        this.eShell.setVisible(false);
        // change tool tip text
        this.label.setToolTipText(I18N.tr(LabelInputText.I18N_DOUBLE_CLICK_TO_EDIT));

        // notify any listeners about the change
        notifyListeners();

    }

    /**
     * Set the margins accordingly to the textDirection and centerVertical
     * settings. Layout of the eInputText field is called in the end. <br />
     * The layout and the size of the shell are re-calculated as well.
     */
    private void setTextInputSizeAndMargins() {

        final GridData gridData = (GridData) this.eInputText.getLayoutData();

        int myLeRitMargin;
        int topBottomMargin;

        if (this.textDirection == SWT.HORIZONTAL) {

            // use the configured left right margins
            myLeRitMargin = this.leftRightMargin;

            // in case we need to centre also vertical, we some calculation for
            // the margin
            if (this.centerVertical) {
                topBottomMargin = centerText(this.size.y);
            } else {
                topBottomMargin = 0;
            }

            // set the value in the grid data
            gridData.widthHint = this.size.x - 2 * this.leftRightMargin;
            gridData.heightHint = this.size.y - 2 * topBottomMargin;

        } else {

            // left right margin is 0 for vertical fields
            myLeRitMargin = 0;

            // calculate the margin
            topBottomMargin = centerText(this.size.x);

            // set the values in the grid data
            gridData.widthHint = this.size.y;
            gridData.heightHint = this.size.x - 2 * topBottomMargin;

        }

        // span across the first two columns
        gridData.horizontalSpan = 2;

        // set the margins
        this.eInputText.setMargins(myLeRitMargin, topBottomMargin, myLeRitMargin, topBottomMargin);

        // set updated grid data
        this.eInputText.setLayoutData(gridData);
        this.eInputText.layout(true);
        this.eInputText.pack();

        Logger.debug(String.format("eInputText width: %1$d, height: %2$d", this.eInputText.getSize().x, //$NON-NLS-1$
                this.eInputText.getSize().y));
        this.eShell.layout(true);

        final Point shellSize = this.eShell.computeSize(SWT.DEFAULT, SWT.DEFAULT);
        this.eShell.setSize(shellSize);

        this.eShell.update();

    }

    /**
     * Can be called to update the label in case of a change of the text. e.g.
     * font size, text length
     */
    private void updateLabel() {

        // update the margins and the size
        setTextInputSizeAndMargins();
        // now draw the label
        draw();

    }

    /**
     * A listener for the font style buttons
     *
     * @author Frank Mnster
     *
     */
    private class MyButtonLister implements LabelButtonListener {

        /**
         * Default constructor
         */
        public MyButtonLister() {
            // intentionally left blank
        }

        @Override
        public void notifyInputButtonClicked(final LabelPartSectionsEnum section, final LabelPartButtonsEnum button) {

            ColorDialog colorDialog;
            RGB colorRgb;
            switch (button) {
            case ALIGN_CENTER:
                LabelInputText.this.eInputText.setAlignment(SWT.CENTER);
                break;
            case ALIGN_LEFT:
                LabelInputText.this.eInputText.setAlignment(SWT.LEFT);
                break;
            case ALIGN_RIGHT:
                LabelInputText.this.eInputText.setAlignment(SWT.RIGHT);
                break;
            case FONT:
                final FontDialog fontDialog = new FontDialog(LabelInputText.this.eShell);
                fontDialog.setEffectsVisible(false);
                fontDialog.setFontList(LabelInputText.this.eInputText.getFont().getFontData());
                final FontData fontData = fontDialog.open();
                if (fontData != null) {
                    final Font guiFont = new Font(GUI.getDisplay(), fontData);
                    LabelInputText.this.eInputText.setFont(guiFont);
                }
                // re-adjust the label margins
                setTextInputSizeAndMargins();
                break;
            case COLOUR:
                colorDialog = new ColorDialog(LabelInputText.this.eShell);
                colorDialog.setRGB(LabelInputText.this.eInputText.getForeground().getRGB());
                colorRgb = colorDialog.open();
                if (colorRgb != null) {
                    final Color color = new Color(GUI.getDisplay(), colorRgb);
                    LabelInputText.this.eInputText.setForeground(color);
                }
                break;
            default:
                // Nothing to here
                Logger.warn(String.format("Called with wrong button %1$s ", button)); //$NON-NLS-1$
                break;
            }
        }
    }

    /**
     * Private MouseListeners for editor buttons
     *
     */
    private class MyEditorButtonMouseListener implements MouseListener {

        /**
         * Creates a now mouse listener for the editor buttons
         */
        public MyEditorButtonMouseListener() {
            // Intentionally left blank
        }

        @Override
        public void mouseDoubleClick(final MouseEvent event) {
            // nothing to do here
        }

        @Override
        public void mouseDown(final MouseEvent event) {
            // nothing to do here
        }

        @Override
        public void mouseUp(final MouseEvent event) {

            // create the runnable with lambda
            final Runnable runnable = () -> {
                if ("SAVE".equals(event.widget.getData(LabelInputText.EDITOR_BUTTON_ID))) { //$NON-NLS-1$
                    // save the text and hide the editor
                    saveTextAndHideEditor();
                } else {
                    // reset the text and hide editor
                    LabelInputText.this.eInputText.setText(LabelInputText.this.text);
                    // hide editor
                    LabelInputText.this.eShell.setVisible(false);
                    // change tool tip text
                    LabelInputText.this.label.setToolTipText(I18N.tr(LabelInputText.I18N_DOUBLE_CLICK_TO_EDIT));
                }
            };
            // start the runnable synchronously
            GUI.getDisplay().syncExec(runnable);
        }

    }
}
