/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.parts.labelpart.buttons;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum;
import de.willuhn.jameica.gui.util.SWTUtil;

/**
 * Button area to select the direction of a label
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class LabelDirectionButtons extends AbstractButtonArea {

    /**
     * Gr&ouml;&szlig;e des T-Buttons
     */
    private static final Point BUTTON_SIZE = new Point(31, 31);

    /**
     * Creates a new button area on the composite.
     *
     * @param parent
     *            the parent composite
     * @param section
     *            the section to which this area applies to. is of type
     *            {@link LabelPartSectionsEnum}
     * @param listener
     *            the {@link LabelButtonListener} to inform in case a button has
     *            been clicked
     */
    public LabelDirectionButtons(final Composite parent, final LabelPartSectionsEnum section,
            final LabelButtonListener listener) {

        super(parent, section, listener);

    }

    /**
     * @see AbstractButtonArea#drawButtons(Composite)
     */
    @Override
    protected final void drawButtons(final Composite composite) {

        final Button textAlignLeft = new Button(this, SWT.PUSH | SWT.CENTER);
        textAlignLeft.setLocation(AbstractButtonArea.MARGIN, AbstractButtonArea.MARGIN);
        textAlignLeft.setSize(LabelDirectionButtons.BUTTON_SIZE);
        textAlignLeft.setImage(SWTUtil.getImage("direction_vertical_up.png")); //$NON-NLS-1$
        textAlignLeft.setData(LabelPartButtonsEnum.VERTICAL_UP);
        textAlignLeft.addSelectionListener(this);

        final Button textAlignCenter = new Button(this, SWT.CENTER);
        textAlignCenter.setLocation(
                textAlignLeft.getBounds().x + textAlignLeft.getBounds().width + AbstractButtonArea.MARGIN,
                AbstractButtonArea.MARGIN);
        textAlignCenter.setSize(LabelDirectionButtons.BUTTON_SIZE);
        textAlignCenter.setImage(SWTUtil.getImage("direction_horizontal.png")); //$NON-NLS-1$
        textAlignCenter.setData(LabelPartButtonsEnum.HORIZONTAL);
        textAlignCenter.addSelectionListener(this);

        final Button textAlignRight = new Button(this, SWT.CENTER);
        textAlignRight.setLocation(
                textAlignCenter.getBounds().x + textAlignCenter.getBounds().width + AbstractButtonArea.MARGIN,
                AbstractButtonArea.MARGIN);
        textAlignRight.setSize(LabelDirectionButtons.BUTTON_SIZE);
        textAlignRight.setImage(SWTUtil.getImage("direction_vertical_down.png")); //$NON-NLS-1$
        textAlignRight.setData(LabelPartButtonsEnum.VERTICAL_DOWN);
        textAlignRight.addSelectionListener(this);

        this.pack();
        this.setSize(this.getSize().x + AbstractButtonArea.MARGIN, this.getSize().y + AbstractButtonArea.MARGIN);

    }

}
