/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.parts.register;

import java.rmi.RemoteException;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.SWTUtil;

/**
 * Setzt Farben und Schriften fr das Register.
 *
 * @author Frank Mnster
 * @since 1.0
 *
 */
public class RegisterSettingsPart implements Part, SelectionListener {

    /**
     * Die max. L&auml;nge des Beschreibungsfeldes
     */
    private static final int CONTENT_LENGTH = 250;

    /**
     * Die Hintergrundfarbe des Index
     */
    private Color indexBgColor;

    /**
     * Der Schrifttype des Index
     */
    private Font indexFont;

    /**
     * Die Hintergrundfarbe der Beschreibung
     */
    private Color contentBgColor;

    /**
     * Der Texttype der Beschreibung
     */
    private Font contentFont;

    /**
     * Der Container in dem alles gezeichnet wird
     */
    private transient Composite container;
    /**
     * Das Indexfeld
     */
    private transient CLabel indexLabel;
    /**
     * Das Beschreibungsfeld
     */
    private transient CLabel contentLabel;

    /**
     * Der Button f&uuml;r den Index-Schrifttyp
     */
    private transient Button indexFontButton;
    /**
     * Der Button f&uuml;r die Index-Hintergrundfarbe
     */
    private transient Button indexBgColorBtn;
    /**
     * Der Button f&uuml;r deb Beschreibungstexttyp
     */
    private transient Button contentFontButton;
    /**
     * Der Button f&uuml;r die Beschreibungstext-Hintergrundfarbe
     */
    private transient Button contentBgColorBtn;

    /**
     * Die Listener fr einen Button-Click-Event
     */
    private final transient Set<RegisterSettingsListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Erstellt ein neues RegisterSettingsPart Object
     *
     * @param listener
     *            RegisterSettingsListener
     */
    public RegisterSettingsPart(final RegisterSettingsListener listener) {
        this.addListener(listener);
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final RegisterSettingsListener listener) {
        this.listeners.add(listener);
    }

    /**
     * Liefert die Hintergrundfarbe der Beschreibung
     *
     * @return die Hintergrundfarbe der Beschreibung
     */
    public Color getContentBgColor() {
        if (this.contentBgColor == null) {
            this.contentBgColor = new Color(GUI.getDisplay(), 240, 240, 240);
        }
        return this.contentBgColor;

    }

    /**
     * Liefert die Schrift der Beschreibung
     *
     * @return die Schrift der Beschreibung
     */
    public Font getContentFont() {
        if (this.contentFont == null) {
            this.contentFont = GUI.getDisplay().getSystemFont();
        }
        return this.contentFont;
    }

    /**
     * Liefert die Hintergrundfarbe des Index
     *
     * @return die Hintergrundfarbe des Index
     */
    public Color getIndexBgColor() {
        if (this.indexBgColor == null) {
            this.indexBgColor = new Color(GUI.getDisplay(), 255, 250, 150);
        }
        return this.indexBgColor;

    }

    /**
     * Liefert die Schrift des Index
     *
     * @return die Schrift des Index
     */
    public Font getIndexFont() {
        if (this.indexFont == null) {
            this.indexFont = GUI.getDisplay().getSystemFont();
        }
        return this.indexFont;
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten
     * Buttons
     *
     * @param buttonObject
     *            mit Button, Schrift- und/oder Farbinformationen
     */
    public final void notifyListeners(final RegisterSettingsButtonObject buttonObject) {

        for (final RegisterSettingsListener listener : this.listeners) {
            listener.notifyButtonClicked(buttonObject);
        }
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        if (this.container != null && !this.container.isDisposed()) {
            this.container.dispose();
        }

        this.container = new Composite(parent, SWT.NONE);
        final GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 7;
        this.container.setLayout(gridLayout);

        updateControls();
        this.container.pack();

    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final RegisterSettingsListener listener) {
        this.listeners.remove(listener);
    }

    /**
     * Setzt die Hintergrundfarbe der Beschreibung
     *
     * @param color
     *            zu setzende Farbe
     */
    public void setContentBgColor(final Color color) {
        if (color != null) {
            this.contentBgColor = color;
            if (this.contentLabel != null) {
                this.contentLabel
                        .setBackground(getLabelBg(this.contentLabel.getSize().x, this.contentLabel.getSize().y, color));
            }
        }
    }

    /**
     * Setzt die Schrift der Beschreibung
     *
     * @param font
     *            zu setzende Schrift
     */
    public void setContentFont(final Font font) {
        if (font != null) {
            this.contentFont = font;
            if (this.contentLabel != null) {
                this.contentLabel.setFont(font);
                updateControls();
            }
        }
    }

    /**
     * Setzt die Hintergrundfarbe des Index
     *
     * @param color
     *            zu setzende Farbe
     */
    public void setIndexBgColor(final Color color) {
        if (color != null) {
            this.indexBgColor = color;
            if (this.indexLabel != null) {
                this.indexLabel
                        .setBackground(getLabelBg(this.indexLabel.getSize().x, this.indexLabel.getSize().y, color));
            }
        }
    }

    /**
     * Setzt die Schrift des Indexes
     *
     * @param font
     *            zu setzende Schrift
     */
    public void setIndexFont(final Font font) {
        if (font != null) {
            this.indexFont = font;
            if (this.indexLabel != null) {
                this.indexLabel.setFont(font);
                updateControls();
            }
        }
    }

    /**
     *
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse
     *      .swt.events.SelectionEvent)
     */
    @Override
    public void widgetDefaultSelected(final SelectionEvent arg0) {
        // hier nix zu tun
    }

    /**
     *
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt
     *      .events.SelectionEvent)
     */
    @Override
    public void widgetSelected(final SelectionEvent evt) {

        final Object source = evt.getSource();
        if (source instanceof Button) {
            final Button button = (Button) source;
            if (this.indexFontButton.equals(button)) {
                final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
                        RegisterSettingsButtonsEnum.INDEX_FONT, this.indexFont);
                notifyListeners(btnObj);
            } else if (this.indexBgColorBtn.equals(button)) {
                final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
                        RegisterSettingsButtonsEnum.INDEX_BACKGROUND_COLOR, this.indexBgColor);
                notifyListeners(btnObj);
            } else if (this.contentFontButton.equals(button)) {
                final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
                        RegisterSettingsButtonsEnum.CONTENT_FONT, this.contentFont);
                notifyListeners(btnObj);
            } else if (this.contentBgColorBtn.equals(button)) {
                final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
                        RegisterSettingsButtonsEnum.CONTENT_BACKGROUND_COLOR, this.contentBgColor);
                notifyListeners(btnObj);
            }
        }
    }

    private Image getLabelBg(final int length, final int height, final Color color) {

        final Image image = new Image(GUI.getDisplay(), length, height);

        final GC myGc = new GC(image);
        myGc.setBackground(color);
        myGc.fillOval(0, 0, height / 2, height);
        myGc.fillRectangle(height / 4, 0, length, height);
        myGc.dispose();

        return image;
    }

    private int getMinLabelWidth(final CLabel label) {

        final GC myGc = new GC(label);

        final FontMetrics fontMetrics = myGc.getFontMetrics();
        final double charWidth = fontMetrics.getAverageCharacterWidth();
        myGc.dispose();
        return (int) (charWidth * 8) + 15;

    }

    private int getMinRowHight() {

        final int rowHightIndex = getIndexFont().getFontData()[0].getHeight();
        final int rowHightContent = getContentFont().getFontData()[0].getHeight();
        final int hight = Math.max(rowHightIndex, rowHightContent);

        return (int) (hight * 1.8);

    }

    private void updateControls() {

        for (final Control child : this.container.getChildren()) {
            child.dispose();
        }

        final int rowHight = getMinRowHight();

        this.indexLabel = new CLabel(this.container, SWT.NONE);
        this.indexLabel.setFont(getIndexFont());
        GridData gridData = new GridData(GridData.VERTICAL_ALIGN_CENTER);
        gridData.widthHint = getMinLabelWidth(this.indexLabel);
        gridData.heightHint = rowHight;
        this.indexLabel.setLayoutData(gridData);

        this.indexLabel.setText(I18N.tr("RegisterSettingsPart.index")); //$NON-NLS-1$
        this.indexLabel.setLeftMargin(15);
        this.indexLabel.setBackgroundImage(getLabelBg(gridData.widthHint, rowHight, getIndexBgColor()));

        this.indexFontButton = new Button(this.container, SWT.PUSH | SWT.CENTER);
        this.indexFontButton.setText(I18N.tr("RegisterSettingsPart.font")); //$NON-NLS-1$
        this.indexFontButton.setToolTipText(I18N.tr("RegisterSettingsPart.fontIndexTooltip")); //$NON-NLS-1$
        this.indexFontButton.addSelectionListener(this);

        this.indexBgColorBtn = new Button(this.container, SWT.PUSH | SWT.CENTER);
        this.indexBgColorBtn.setImage(SWTUtil.getImage("colors-icon.png")); //$NON-NLS-1$
        this.indexBgColorBtn.setToolTipText(I18N.tr("RegisterSettingsPart.bgColorIndexTooltip")); //$NON-NLS-1$
        this.indexBgColorBtn.addSelectionListener(this);

        final Label leerLabel = new Label(this.container, SWT.NONE);
        leerLabel.setText("     "); //$NON-NLS-1$

        this.contentLabel = new CLabel(this.container, SWT.NONE);
        gridData = new GridData(GridData.VERTICAL_ALIGN_CENTER);
        gridData.widthHint = RegisterSettingsPart.CONTENT_LENGTH;
        gridData.heightHint = rowHight;
        this.contentLabel.setLayoutData(gridData);
        this.contentLabel.setText(I18N.tr("RegisterSettingsPart.description")); //$NON-NLS-1$
        this.contentLabel.setLeftMargin(15);
        this.contentLabel.setFont(getContentFont());
        this.contentLabel
                .setBackgroundImage(getLabelBg(RegisterSettingsPart.CONTENT_LENGTH, rowHight, getContentBgColor()));

        this.contentFontButton = new Button(this.container, SWT.PUSH | SWT.CENTER);
        this.contentFontButton.setText(I18N.tr("RegisterSettingsPart.font")); //$NON-NLS-1$
        this.contentFontButton.setToolTipText(I18N.tr("RegisterSettingsPart.fontDescriptionTooltip")); //$NON-NLS-1$
        this.contentFontButton.addSelectionListener(this);

        this.contentBgColorBtn = new Button(this.container, SWT.PUSH | SWT.CENTER);
        this.contentBgColorBtn.setImage(SWTUtil.getImage("colors-icon.png")); //$NON-NLS-1$
        this.contentBgColorBtn.setToolTipText(I18N.tr("RegisterSettingsPart.bgColorDescriptionTooltip")); //$NON-NLS-1$
        this.contentBgColorBtn.addSelectionListener(this);

        this.container.pack();
        this.container.layout();
        this.container.getParent().layout();
        // Layout new malen (ganz schn weit oben :-) )
        this.container.getParent().getParent().getParent().layout();

    }

}
