/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;

import org.eclipse.nebula.paperclips.core.EmptyPrint;
import org.eclipse.nebula.paperclips.core.LinePrint;
import org.eclipse.nebula.paperclips.core.Print;
import org.eclipse.nebula.paperclips.core.SidewaysPrint;
import org.eclipse.nebula.paperclips.core.border.BorderPrint;
import org.eclipse.nebula.paperclips.core.border.LineBorder;
import org.eclipse.nebula.paperclips.core.grid.DefaultGridLook;
import org.eclipse.nebula.paperclips.core.grid.GridPrint;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.nebula.paperclips.core.text.TextPrint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPart;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelSizeEnum;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.frankmuenster.jameica.orverw.util.Utils;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 */
public class PrintLabel extends AbstractPrint {

    /**
     * Zero width column for height spacing
     */
    private static final String GRID_0MM_COLUMN = "0mm, "; //$NON-NLS-1$

    /**
     * Der Ordner dessen Label gedruckt werden soll
     */
    private final Ordner ordner; // NOPMD

    /**
     * Der Titel
     */
    private final String titel; // NOPMD

    /**
     * Die Gre des Labels
     */
    private final LabelSizeEnum size; // NOPMD

    /**
     * die Hintergrundfarbe
     */
    private final RGB lblBgColor; // NOPMD

    /**
     * Der Konstruktor
     *
     * @param ordner
     *            der Ordner dessen Label gedruckt werden soll
     * @throws ApplicationException
     */
    public PrintLabel(final Ordner ordner) throws ApplicationException {

        // Call the super class
        super();
        // store the
        this.ordner = ordner;
        try {
            this.size = LabelSizeEnum.getLabelSize(ordner.getWidth());
            this.lblBgColor = ordner.getLabelBackground().getRGB();
            this.titel = ordner.getTitleText();
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("PrintLabel.errorCreatingPrintObj"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see AbstractPrint#getTitle()
     */
    @Override
    public String getTitle() {

        return Utils.format(this.titel, 10);
    }

    /**
     * @see AbstractPrint#customise(PagePrint)
     */
    @Override
    protected void customise(final PagePrint page) throws ApplicationException {

        // hier brauchen wir nix tun tun
    }

    /**
     * @see AbstractPrint#getPrintContent()
     */
    @Override
    @SuppressWarnings("PMD.ExcessiveMethodLength")
    protected Print getPrintContent() throws ApplicationException {

        // crate the gridlook
        final DefaultGridLook look = new DefaultGridLook();
        look.setBodyBackground(this.lblBgColor);
        // calculate padding depending on the width
        look.setCellPadding(LabelPart.TEXT_LEFT_RIGHT_MARGIN, 0);

        final DefaultGridLook textSections4IndentLook = new DefaultGridLook();
        textSections4IndentLook.setBodyBackground(this.lblBgColor);
        textSections4IndentLook.setCellPadding(LabelPart.TEXT_LEFT_RIGHT_MARGIN, 0);
        final DefaultGridLook textSections0IndentLook = new DefaultGridLook();
        textSections0IndentLook.setBodyBackground(this.lblBgColor);
        textSections0IndentLook.setCellPadding(0, 0);

        // set with removed by the padding
        final int width = (int) (this.size.getWidth() * 0.039 * 72);
        final int topHeight = (int) (LabelPartSectionsEnum.TOP.getHeight() * 0.039 * 72);
        final int barHeight = (int) (LabelPartSectionsEnum.BAR.getHeight() * 0.039 * 72);
        final int titleHeight = (int) (LabelPartSectionsEnum.TITLE.getHeight() * 0.039 * 72);
        final int contentHeight = (int) (LabelPartSectionsEnum.CONTENT.getHeight() * 0.039 * 72);
        final int footerHeight = (int) (LabelPartSectionsEnum.FOOTER.getHeight() * 0.039 * 72);
        final int idHeight = (int) (8 * 0.039 * 72);
        final GridPrint outer = new GridPrint(String.valueOf(width + 10));
        final GridPrint grid = new GridPrint(String.valueOf(width), look);
        try {
            // top to have a nice margin
            final EmptyPrint topSection = new EmptyPrint(width, topHeight);

            // Bar1
            final GridPrint barSection = new GridPrint(PrintLabel.GRID_0MM_COLUMN + width);
            final EmptyPrint barEmpty = new EmptyPrint(0, barHeight);
            barSection.add(barEmpty);
            final LinePrint barLine = new LinePrint(SWT.HORIZONTAL, (float) 4.0);
            if (this.ordner.isBarVisible()) {
                barLine.setRGB(this.ordner.getBarColor().getRGB());
            } else {
                barLine.setRGB(this.lblBgColor);
            }
            barSection.add(SWT.CENTER, SWT.CENTER, barLine);

            // Title
            final GridPrint titleSection = new GridPrint(PrintLabel.GRID_0MM_COLUMN + width);
            titleSection.setLook(
                    this.ordner.getTitleAlignment() == SWT.CENTER ? textSections0IndentLook : textSections4IndentLook);
            final EmptyPrint titleEmpty = new EmptyPrint(0, titleHeight);
            titleSection.add(titleEmpty);

            final TextPrint titleTextPrint = new TextPrint(this.ordner.getTitleText(),
                    this.ordner.getTitleFont().getFontData()[0], this.ordner.getTitleAlignment());
            titleTextPrint.setForeground(this.ordner.getTitleColor().getRGB());
            titleTextPrint.setWordSplitting(true);
            addTextToSection(titleSection, LabelPartSectionsEnum.TITLE, titleTextPrint, this.ordner.getTitleAlignment(),
                    this.ordner.getTitleDirection());

            // Content
            final GridPrint contentSection = new GridPrint(PrintLabel.GRID_0MM_COLUMN + width);
            contentSection.setLook(this.ordner.getContentAlignment() == SWT.CENTER ? textSections0IndentLook
                    : textSections4IndentLook);
            final EmptyPrint contentEmpty = new EmptyPrint(0, contentHeight);
            contentSection.add(contentEmpty);

            final TextPrint contentTextPrint = new TextPrint(this.ordner.getContent(),
                    this.ordner.getContentFont().getFontData()[0], this.ordner.getContentAlignment());
            contentTextPrint.setForeground(this.ordner.getContentColor().getRGB());
            contentTextPrint.setWordSplitting(true);
            addTextToSection(contentSection, LabelPartSectionsEnum.CONTENT, contentTextPrint,
                    this.ordner.getContentAlignment(), this.ordner.getContentDirection());

            // Footer
            final GridPrint footerSection = new GridPrint(PrintLabel.GRID_0MM_COLUMN + width);
            footerSection.setLook(
                    this.ordner.getFooterAlignment() == SWT.CENTER ? textSections0IndentLook : textSections4IndentLook);

            final EmptyPrint footerEmpty = new EmptyPrint(0, footerHeight);
            footerSection.add(footerEmpty);

            final TextPrint footerTextPrint = new TextPrint(this.ordner.getFooter(),
                    this.ordner.getFooterFont().getFontData()[0], this.ordner.getFooterAlignment());
            footerTextPrint.setForeground(this.ordner.getFooterColor().getRGB());
            footerTextPrint.setWordSplitting(true);
            addTextToSection(footerSection, LabelPartSectionsEnum.FOOTER, footerTextPrint,
                    this.ordner.getFooterAlignment(), this.ordner.getFooterDirection());

            // ID
            final GridPrint idSection = new GridPrint(width + ", 0mm", look); //$NON-NLS-1$
            final EmptyPrint idEmpty = new EmptyPrint(0, idHeight);
            final TextPrint idTextPrint = new TextPrint("ID " + this.ordner.getID(), //$NON-NLS-1$
                    new FontData("Arial", 10, SWT.NORMAL), SWT.RIGHT); //$NON-NLS-1$
            if (this.ordner.isIdVisible()) {
                idSection.add(SWT.RIGHT, SWT.TOP, idTextPrint);
            }
            idSection.add(idEmpty);

            // add all pieces to the inner grid
            grid.add(topSection);
            grid.add(barSection);
            grid.add(titleSection);
            grid.add(barSection);
            grid.add(contentSection);
            grid.add(footerSection);
            grid.add(idSection);

            // add a line to the order grid
            final LineBorder lineBorder = new LineBorder();
            lineBorder.setGapSize(1);
            lineBorder.setLineWidth(1);
            final BorderPrint gridBorderPrint = new BorderPrint(grid, lineBorder);
            outer.add(gridBorderPrint);
        } catch (

        final RemoteException e) {
            throw new ApplicationException(I18N.tr("PrintLabel.errorCreatingPrintContent", this.titel), e); //$NON-NLS-1$
        }
        return outer;
    }

    /**
     * Adds the title correctly rotated to the GrindPrint
     *
     * @param textPrint
     * @param parent
     *
     * @param child
     *            the Gridprint to add the title
     * @param width
     * @throws RemoteException
     *             in case the data cannot be read form the ordner
     */
    private void addTextToSection(final GridPrint parent, final LabelPartSectionsEnum labelSection,
            final TextPrint textPrint, final int textAlignment, final int textDirection) throws RemoteException {

        int hAlign;
        int vAlign;

        Print print = textPrint;

        if (textDirection == SWT.HORIZONTAL) {
            // not so much to do
            hAlign = textAlignment;
            if (labelSection == LabelPartSectionsEnum.TITLE) {
                // Vertical align the text
                vAlign = SWT.CENTER;
            } else {
                // place on top
                vAlign = SWT.TOP;
            }
        } else {

            // create a new grid look
            final DefaultGridLook look = new DefaultGridLook();
            look.setBodyBackground(this.lblBgColor);
            look.setCellPadding(0, 10);
            final GridPrint grid = new GridPrint(labelSection.getHeight() + "mm", look); //$NON-NLS-1$

            // if vertical print we always centre horizontal
            hAlign = SWT.CENTER;

            // now for vertical align
            switch (textAlignment) {
            case SWT.LEFT:
                vAlign = SWT.BOTTOM;
                break;
            case SWT.RIGHT:
                vAlign = SWT.TOP;
                break;
            case SWT.CENTER:
                vAlign = SWT.CENTER;
                break;
            default:
                vAlign = SWT.CENTER;
                break;
            }
            grid.add(textAlignment, SWT.CENTER, textPrint);

            // rotate
            final int angle = textDirection == SWT.LEFT ? 90 : -90;
            print = new SidewaysPrint(grid, angle);

        }

        // finally add the text
        parent.add(hAlign, vAlign, print);

    }

}
