/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.print;

import org.eclipse.nebula.paperclips.core.Margins;
import org.eclipse.nebula.paperclips.core.PaperClips;
import org.eclipse.nebula.paperclips.core.Print;
import org.eclipse.nebula.paperclips.core.PrintJob;
import org.eclipse.nebula.paperclips.widgets.PrintPreview;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.printing.PrintDialog;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.util.ApplicationException;

/**
 * Provides a print preview of the label and register
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class PrintPreviewDialog {

    /**
     *
     */
    private static final int ONE_PAGE = 1;

    /**
     * Unser Printjob
     */
    private PrintJob job; // NOPMD

    /**
     * Das Page Objekt zur Druckvorschau
     */
    private final Print page; // NOPMD

    /**
     * Die Dialog-Parent-Shell
     */
    private final Shell parentShell; // NOPMD

    /**
     * Die Shell fr den Dialog
     */
    private Shell shell; // NOPMD

    /**
     * Der Seitentitel, kommt aus dem Ordner Titelfeld
     */
    private final String titel; // NOPMD

    /**
     * Standardkonstruktor
     *
     * @param print
     * @param parentShell
     * @throws ApplicationException
     */
    public PrintPreviewDialog(final AbstractPrint print, final Shell parentShell) throws ApplicationException {

        this.titel = print.getTitle();
        this.page = print.createPrint();
        this.parentShell = parentShell;
    }

    /**
     * &Ouml;ffent die Druckvorschau in einem Dialogfenster.
     *
     * @throws ApplicationException
     */
    public void open() throws ApplicationException {

        this.shell = new Shell(this.parentShell, SWT.SHELL_TRIM | SWT.RESIZE | SWT.APPLICATION_MODAL);
        this.shell.setText(I18N.tr("PrintPreviewDialog.printPreview")); //$NON-NLS-1$
        this.shell.setBounds(100, 100, 830, 630);
        this.job = new PrintJob(I18N.tr("PrintPreviewDialog.printJobTitlePrefix") + this.titel, this.page); //$NON-NLS-1$
        final Margins margins = this.job.getMargins();
        margins.left = 100;
        margins.right = 50;
        this.shell.setLayout(new GridLayout());
        final Composite buttonPanel = new Composite(this.shell, SWT.NONE);
        buttonPanel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        buttonPanel.setLayout(new RowLayout(SWT.HORIZONTAL));
        final PrintPreview preview = new PrintPreview(this.shell, SWT.BORDER);
        preview.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        preview.setFitHorizontal(true);
        preview.setFitVertical(true);
        preview.setPrintJob(this.job);

        this.addButtons(buttonPanel, preview);

        this.shell.setVisible(true);
        while (!this.shell.isDisposed()) {
            if (!this.shell.getDisplay().readAndDispatch()) {
                this.shell.getDisplay().sleep();
            }
        }
    }

    /**
     * Hier wird gedruckt
     */
    protected void handlePrint() {

        final PrintDialog dialog = new PrintDialog(this.shell, SWT.NONE);
        final PrinterData printerData = dialog.open();
        if (printerData != null) {
            // Erstelle Printjob
            // Drucken starten
            PaperClips.print(this.job, printerData);
            // ...und wir schlieen die Vorschau
            this.shell.dispose();
        }
    }

    /**
     * Creates previous/next print and close buttons
     *
     * @param buttonPanel
     *            the panel to print the buttons in
     * @param preview
     *            the printPreview
     */
    private void addButtons(final Composite buttonPanel, final PrintPreview preview) {

        if (preview.getPageCount() > PrintPreviewDialog.ONE_PAGE) {

            final Button prev = new Button(buttonPanel, SWT.PUSH);
            prev.setText(I18N.tr("PrintPreviewDialog.previous")); //$NON-NLS-1$
            prev.addListener(SWT.Selection, new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
                 */
                @Override
                public void handleEvent(final Event event) {

                    preview.setPageIndex(Math.max(preview.getPageIndex() - 1, 0));
                }
            });

            final Button next = new Button(buttonPanel, SWT.PUSH);
            next.setText(I18N.tr("PrintPreviewDialog.next")); //$NON-NLS-1$
            next.addListener(SWT.Selection, new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
                 */
                @Override
                public void handleEvent(final Event event) {

                    preview.setPageIndex(Math.min(preview.getPageIndex() + 1, preview.getPageCount() - 1));
                }
            });
        }

        final Button print = new Button(buttonPanel, SWT.PUSH);
        print.setText(I18N.tr("_print")); //$NON-NLS-1$
        print.addListener(SWT.Selection, new Listener() {

            /**
             * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
             */
            @Override
            public void handleEvent(final Event event) {

                PrintPreviewDialog.this.handlePrint();
            }
        });
        final Button cancel = new Button(buttonPanel, SWT.PUSH);
        cancel.setText(I18N.tr("_close")); //$NON-NLS-1$
        cancel.addListener(SWT.Selection, new Listener() {

            /**
             * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
             */
            @Override
            public void handleEvent(final Event event) {

                preview.dispose();
                // Das Drucken behandeln wir in eine extra Funktion
                PrintPreviewDialog.this.shell.dispose();
            }
        });

    }
}
