/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.util;

import de.willuhn.jameica.gui.GUI;

/**
 * Utility KLasse
 *
 * @author Frank Mnster
 */
public final class Utils {

    /**
     * Factor to convert 72dpi to millimeter
     */
    public static final double POINTS_PER_MM = 2.83464567;

    /**
     * Ersetzt Zeilenumbr&uuml;che durch <b>/</b>.
     *
     * @param text
     *            Den zu formatierenden Text.
     * @param length
     *            Die maximale L&auml;nge des R&uuml;ckgabetextes.
     * @return Den formatierten Text.
     */
    public static String format(final String text, final int length) {

        String result;

        result = text.replaceAll("\r\n", "\n").replaceAll("\\\n", " / "); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

        if (result.startsWith(" / ")) { //$NON-NLS-1$
            result = result.substring(3);
        }
        if (result.endsWith(" / ")) { //$NON-NLS-1$
            result = result.substring(0, result.length() - 3);
        }

        if (result.length() > length - 3) {
            result = result.substring(0, length - 3) + "..."; //$NON-NLS-1$
        }
        return result;

    }

    /**
     * Calculates points on teh display out of millimetre.
     *
     * @param millimeter
     * @return display points
     */
    public static int mmToDisplayPoints(final int millimeter) {

        final int dpi = GUI.getDisplay().getDPI().x;
        final double inch = millimeter / 25.4;
        return (int) (dpi * inch);
    }

    /**
     * Calculates points (72 per inch) out of millimetre.
     *
     * @param millimeter
     * @return points (72 per inch based)
     */
    public static int mmToPoints(final int millimeter) {

        return (int) (millimeter * Utils.POINTS_PER_MM);
    }

    private Utils() {

        // Utility class must be final and have a private constructor
    }
}
